@@ -1,7 +1,7 @@
 /*
  * config_mgr.h - class configManager, a class for managing LMMS-configuration
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -157,6 +157,25 @@ public:
 		return( m_flDir );
 	}
 
+	const QString & ladspaDir( void ) const
+	{
+		return( m_ladDir );
+	}
+
+#ifdef HAVE_STK_H
+	const QString & stkDir( void ) const
+	{
+		return( m_stkDir );
+	}
+#endif
+
+	inline const QStringList & recentlyOpenedProjects( void ) const
+	{
+		return( m_recentlyOpenedProjects );
+	}
+
+	void addRecentlyOpenedProject( const QString & _file );
+
 	const QString & value( const QString & _class,
 					const QString & _attribute ) const;
 	void setValue( const QString & _class, const QString & _attribute,
@@ -171,6 +190,8 @@ public slots:
 	void setVSTDir( const QString & _vd );
 	void setArtworkDir( const QString & _ad );
 	void setFLDir( const QString & _fd );
+	void setLADSPADir( const QString & _fd );
+	void setSTKDir( const QString & _fd );
 
 
 protected slots:
@@ -209,6 +230,12 @@ private:
 	QString m_pluginDir;
 	QString m_vstDir;
 	QString m_flDir;
+	QString m_ladDir;
+#ifdef HAVE_STK_H
+	QString m_stkDir;
+#endif
+	QStringList m_recentlyOpenedProjects;
+
 
 	typedef vvector<QPair<QString, QString> > stringPairVector;
 	typedef QMap<QString, stringPairVector> settingsMap;
