@@ -1,7 +1,7 @@
 /*
  * lcd_spinbox.h - class lcdSpinBox, an improved QLCDNumber
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -30,8 +30,8 @@
 
 #ifdef QT4
 
-#include <QLCDNumber>
-#include <QMap>
+#include <QtCore/QMap>
+#include <QtGui/QLCDNumber>
 
 #else
 
@@ -41,24 +41,22 @@
 #endif
 
 
+#include "automatable_object.h"
+
+
 class QLabel;
 
 
-class lcdSpinBox : public QWidget
+class lcdSpinBox : public QWidget, public automatableObject<int>
 {
 	Q_OBJECT
 public:
-	lcdSpinBox( int _min, int _max, int _num_digits, QWidget * _parent );
+	lcdSpinBox( int _min, int _max, int _num_digits, QWidget * _parent,
+					const QString & _name, track * _track );
 	virtual ~lcdSpinBox();
 
-	void setStep( int _step );
+	virtual void setStep( const int _step );
 
-	inline int value( void ) const
-	{
-		return( m_number->intValue() );
-	}
-
-	void setValue( int _value );
 	void setLabel( const QString & _txt );
 
 	inline void addTextForValue( int _val, const QString & _text )
@@ -68,10 +66,12 @@ public:
 
 
 public slots:
+	virtual void setValue( const int _value );
 	virtual void setEnabled( bool _on );
 
 
 protected:
+	virtual void contextMenuEvent( QContextMenuEvent * _me );
 	virtual void mousePressEvent( QMouseEvent * _me );
 	virtual void mouseMoveEvent( QMouseEvent * _me );
 	virtual void mouseReleaseEvent( QMouseEvent * _me );
@@ -81,11 +81,6 @@ protected:
 private:
 	QMap<int, QString> m_textForValue;
 
-	int m_value;
-	int m_minValue;
-	int m_maxValue;
-	int m_step;
-
 	QLCDNumber * m_number;
 	QLabel * m_label;
 
@@ -94,6 +89,7 @@ private:
 
 signals:
 	void valueChanged( int );
+	void manualChange( void );
 
 } ;
 
