@@ -1,7 +1,7 @@
 /*
  * level_object.h - declaration of class levelObject
  *
- * Copyright (c) 2006 Javier Serrano Polo <jasp00/at/users.sourceforge.net>
+ * Copyright (c) 2006-2007 Javier Serrano Polo <jasp00/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -26,8 +26,6 @@
 #ifndef _LEVEL_OBJECT_H
 #define _LEVEL_OBJECT_H
 
-#include "midi_time.h"
-
 
 
 
@@ -50,9 +48,14 @@ public:
 
 	virtual void setLevel( int _level ) = 0;
 
-	virtual QString levelToLabel( int _level ) = 0;
+	virtual QString levelToLabel( int _level ) const = 0;
 	virtual int labelToLevel( QString _label ) = 0;
 
+	virtual QString displayName( void ) const
+	{
+		return( NULL );
+	}
+
 
 protected:
 	int m_minLevel;
