@@ -1,7 +1,7 @@
 /*
  * mixer.h - audio-device-independent mixer for LMMS
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -35,24 +35,23 @@
 #ifdef QT4
 
 #include <QtCore/QMutex>
+#include <QtCore/QThread>
 #include <QtCore/QVector>
 
 #else
 
 #include <qobject.h>
 #include <qmutex.h>
+#include <qthread.h>
 #include <qvaluevector.h>
 
 #endif
 
 
 #include "types.h"
-#include "volume.h"
-#include "panning.h"
 #include "note.h"
 #include "play_handle.h"
-#include "effect_board.h"
-#include "engine.h"
+#include "fifo_buffer.h"
 
 
 class audioDevice;
@@ -60,7 +59,7 @@ class midiClient;
 class audioPort;
 
 
-const fpab_t DEFAULT_BUFFER_SIZE = 512;
+const fpp_t DEFAULT_BUFFER_SIZE = 512;
 
 const ch_cnt_t DEFAULT_CHANNELS = 2;
 
@@ -106,12 +105,12 @@ const octaves BASE_OCTAVE = OCTAVE_4;
 
 
 
-class mixer : public QObject, public engineObject
+class mixer : public QObject
 {
 	Q_OBJECT
 public:
 	void initDevices( void );
-	void FASTCALL clear( const bool _everything = FALSE );
+	void FASTCALL clear( void );
 
 
 	// audio-device-stuff
@@ -131,13 +130,14 @@ public:
 	// audio-port-stuff
 	inline void addAudioPort( audioPort * _port )
 	{
-		pause();
+		lock();
 		m_audioPorts.push_back( _port );
-		play();
+		unlock();
 	}
 
 	inline void removeAudioPort( audioPort * _port )
 	{
+		lock();
 		vvector<audioPort *>::iterator it = qFind( m_audioPorts.begin(),
 							m_audioPorts.end(),
 							_port );
@@ -145,6 +145,7 @@ public:
 		{
 			m_audioPorts.erase( it );
 		}
+		unlock();
 	}
 
 
@@ -165,35 +166,39 @@ public:
 	{
 		if( criticalXRuns() == FALSE )
 		{
+			lockPlayHandles();
 			m_playHandles.push_back( _ph );
+			unlockPlayHandles();
 			return( TRUE );
 		}
 		delete _ph;
 		return( FALSE );
 	}
 
-	inline void removePlayHandle( playHandle * _ph )
+	inline void removePlayHandle( const playHandle * _ph )
 	{
+		lockPlayHandlesToRemove();
 		m_playHandlesToRemove.push_back( _ph );
+		unlockPlayHandlesToRemove();
 	}
 
-	inline const playHandleVector & playHandles( void ) const
+	inline playHandleVector & playHandles( void )
 	{
 		return( m_playHandles );
 	}
 
-	void checkValidityOfPlayHandles( void );
+	void removePlayHandles( track * _track );
 
-	inline bool haveNoRunningNotes( void ) const
+	inline bool hasPlayHandles( void ) const
 	{
-		return( m_playHandles.size() == 0 );
+		return( !m_playHandles.empty() );
 	}
 
 
 	// methods providing information for other classes
-	inline fpab_t framesPerAudioBuffer( void ) const
+	inline fpp_t framesPerPeriod( void ) const
 	{
-		return( m_framesPerAudioBuffer );
+		return( m_framesPerPeriod );
 	}
 
 	inline const surroundSampleFrame * currentAudioBuffer( void ) const
@@ -244,81 +249,112 @@ public:
 	}
 
 
-	// methods for controlling mixer-state
-	void pause( void )
+	// methods needed by other threads to alter knob values, waveforms, etc
+	void lock( void )
 	{
-		if( m_mixMutexLockLevel == 0 )
-		{
-			m_mixMutex.lock();
-		}
-		++m_mixMutexLockLevel;
+		m_globalMutex.lock();
 	}
 
-	void play( void )
+	void unlock( void )
 	{
-		if( m_mixMutexLockLevel == 1 )
-		{
-			m_mixMutex.unlock();
-		}
-		--m_mixMutexLockLevel;
+		m_globalMutex.unlock();
 	}
 
+	void lockPlayHandles( void )
+	{
+		m_playHandlesMutex.lock();
+	}
+
+	void unlockPlayHandles( void )
+	{
+		m_playHandlesMutex.unlock();
+	}
+
+	void lockPlayHandlesToRemove( void )
+	{
+		m_playHandlesToRemoveMutex.lock();
+	}
+
+	void unlockPlayHandlesToRemove( void )
+	{
+		m_playHandlesToRemoveMutex.unlock();
+	}
 
 	// audio-buffer-mgm
 	void FASTCALL bufferToPort( const sampleFrame * _buf,
-					const fpab_t _frames,
-					const fpab_t _framesAhead,
+					const fpp_t _frames,
+					const f_cnt_t _offset,
 					const volumeVector & _volumeVector,
 							audioPort * _port );
 
 	void FASTCALL clearAudioBuffer( sampleFrame * _ab,
-							const f_cnt_t _frames );
+						const f_cnt_t _frames,
+						const f_cnt_t _offset = 0 );
 #ifndef DISABLE_SURROUND
 	void FASTCALL clearAudioBuffer( surroundSampleFrame * _ab,
-							const f_cnt_t _frames );
+						const f_cnt_t _frames,
+						const f_cnt_t _offset = 0 );
 #endif
 
 	bool criticalXRuns( void ) const;
 
-	const surroundSampleFrame * renderNextBuffer( void );
+	const surroundSampleFrame * nextBuffer( void )
+	{
+		return( m_fifo->read() );
+	}
 
 
 public slots:
 	void setHighQuality( bool _hq_on = FALSE );
 	void setClipScaling( bool _state );
 
+
 signals:
 	void sampleRateChanged( void );
 	void nextAudioBuffer( const surroundSampleFrame *, int _frames );
 
 
 private:
-	mixer( engine * _engine );
-	~mixer();
+	typedef fifoBuffer<surroundSampleFrame *> fifo;
+
+	class fifoWriter : public QThread
+	{
+	public:
+		fifoWriter( mixer * _mixer, fifo * _fifo );
+
+		void finish( void );
+
+
+	private:
+		mixer * m_mixer;
+		fifo * m_fifo;
+		bool m_writing;
+
+		virtual void run( void );
+
+	} ;
+
+
+	mixer( void );
+	virtual ~mixer();
 
 	void startProcessing( void );
 	void stopProcessing( void );
 
 
-	// we don't allow to create mixer by using copy-ctor
-	mixer( const mixer & ) :
-		engineObject( NULL )
-	{
-	}
-
-
-
 	audioDevice * tryAudioDevices( void );
 	midiClient * tryMIDIClients( void );
 
 	void processBuffer( const surroundSampleFrame * _buf,
 						const fx_ch_t _fx_chnl );
 
-	void FASTCALL scaleClip( fpab_t _frame, ch_cnt_t _chnl );
+	void FASTCALL scaleClip( fpp_t _frame, ch_cnt_t _chnl );
+
+	const surroundSampleFrame * renderNextBuffer( void );
 
 	vvector<audioPort *> m_audioPorts;
 
-	fpab_t m_framesPerAudioBuffer;
+	fpp_t m_framesPerPeriod;
 
 	surroundSampleFrame * m_readBuf;
 	surroundSampleFrame * m_writeBuf;
@@ -328,19 +364,20 @@ private:
 	Uint8 m_writeBuffer;
 	Uint8 m_analBuffer;
 	Uint8 m_poolDepth;
-	
+
 	bool m_scaleClip;
 	surroundSampleFrame m_maxClip;
 	surroundSampleFrame m_previousSample;
-	fpab_t m_halfStart[SURROUND_CHANNELS];
+	fpp_t m_halfStart[SURROUND_CHANNELS];
 	bool m_clipped[SURROUND_CHANNELS];
 	bool m_oldBuffer[SURROUND_CHANNELS];
 	bool m_newBuffer[SURROUND_CHANNELS];
 	
 	Uint8 m_cpuLoad;
+	int m_parallelizingLevel;
 
 	playHandleVector m_playHandles;
-	playHandleVector m_playHandlesToRemove;
+	constPlayHandleVector m_playHandlesToRemove;
 
 	qualityLevels m_qualityLevel;
 	float m_masterGain;
@@ -355,8 +392,13 @@ private:
 	QString m_midiClientName;
 
 
-	QMutex m_mixMutex;
-	Uint8 m_mixMutexLockLevel;
+	QMutex m_globalMutex;
+	QMutex m_playHandlesMutex;
+	QMutex m_playHandlesToRemoveMutex;
+
+
+	fifo * m_fifo;
+	fifoWriter * m_fifo_writer;
 
 
 	friend class engine;
