@@ -1,7 +1,7 @@
 /*
  * mmp.h - class for reading and writing multimedia-project-files
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -39,6 +39,11 @@
 #endif
 
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif 
+
+
 #include "types.h"
 
 
@@ -52,6 +57,7 @@ public:
 		SONG_PROJECT_TEMPLATE,
 		INSTRUMENT_TRACK_SETTINGS,
 		DRAG_N_DROP_DATA,
+		CLIPBOARD_DATA,
 		JOURNAL_DATA,
 		EFFECT_SETTINGS,
 		VIDEO_PROJECT,		// might come later...
@@ -62,12 +68,12 @@ public:
 
 
 	multimediaProject( const QString & _in_file_name,
-						bool _is_filename = TRUE );
+						bool _is_filename = TRUE,
+						bool _upgrade = TRUE );
 	multimediaProject( projectTypes _project_type );
 	~multimediaProject();
 
-	bool FASTCALL writeFile( const QString & _fn,
-						bool _overwrite_check = TRUE );
+	bool FASTCALL writeFile( QString & _fn, bool _overwrite_check = TRUE );
 
 	inline QDomElement & content( void )
 	{
@@ -92,6 +98,8 @@ private:
 
 	void cleanMetaNodes( QDomElement _de );
 
+	void upgrade( void );
+
 
 	struct typeDescStruct
 	{
