@@ -2,7 +2,7 @@
  * note.h - declaration of class note which contains all informations about a
  *          note + definitions of several constants and enums
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -35,7 +35,7 @@
 
 #ifdef QT4
 
-#include <QVector>
+#include <QtCore/QVector>
 
 #else
 
@@ -46,7 +46,10 @@
 #include "volume.h"
 #include "panning.h"
 #include "midi_time.h"
-#include "settings.h"
+#include "journalling_object.h"
+
+class detuningHelper;
+
 
 enum tones
 {
@@ -85,78 +88,130 @@ const octaves MAX_OCTAVE = OCTAVE_8;
 const int WHITE_KEYS_PER_OCTAVE	= 7;
 const int BLACK_KEYS_PER_OCTAVE	= 5;
 const int NOTES_PER_OCTAVE = WHITE_KEYS_PER_OCTAVE + BLACK_KEYS_PER_OCTAVE;
-const int OCTAVES = 9;
+const int OCTAVES = MAX_OCTAVE+1;
+const int NOTES = OCTAVES*NOTES_PER_OCTAVE;
 
 
 
-class note : public settings
+class note : public journallingObject
 {
 public:
-	note( const midiTime & _length = 0, const midiTime & _pos = 0,
-		tones _tone = A, octaves _octave = DEFAULT_OCTAVE,
+	note( const midiTime & _length = 0,
+		const midiTime & _pos = 0,
+		tones _tone = A,
+		octaves _octave = DEFAULT_OCTAVE,
 		volume _volume = DEFAULT_VOLUME,
-		panning _panning = DEFAULT_PANNING ) FASTCALL;
-	~note();
+		panning _panning = DEFAULT_PANNING,
+		detuningHelper * _detuning = NULL ) FASTCALL;
+	note( const note & _note );
+	virtual ~note();
+
 	void FASTCALL setLength( const midiTime & _length );
 	void FASTCALL setPos( const midiTime & _pos );
-	void FASTCALL setTone( tones _tone = C );
-	void FASTCALL setOctave( octaves _octave = DEFAULT_OCTAVE );
-	void FASTCALL setKey( int _key );
-	void FASTCALL setVolume( volume _volume = DEFAULT_VOLUME );
-	void FASTCALL setPanning( panning _panning = DEFAULT_PANNING );
+	void FASTCALL setTone( const tones _tone = C );
+	void FASTCALL setOctave( const octaves _octave = DEFAULT_OCTAVE );
+	void FASTCALL setKey( const int _key );
+	void FASTCALL setVolume( const volume _volume = DEFAULT_VOLUME );
+	void FASTCALL setPanning( const panning _panning = DEFAULT_PANNING );
+	void FASTCALL quantizeLength( const int _q_grid );
+	void FASTCALL quantizePos( const int _q_grid );
+
 	inline midiTime endPos( void ) const
 	{
 		return( m_pos + m_length);
 	}
+
 	inline const midiTime & length( void ) const
 	{
 		return( m_length );
 	}
+
 	inline const midiTime & pos( void ) const
 	{
 		return( m_pos );
 	}
+
 	inline midiTime pos( midiTime _base_pos ) const
 	{
 		return( m_pos - _base_pos );
 	}
+
 	inline tones tone( void ) const
 	{
 		return( m_tone );
 	}
+
 	inline octaves octave( void ) const
 	{
 		return( m_octave );
 	}
+
 	inline int key( void ) const
 	{
 		return( m_octave * NOTES_PER_OCTAVE + m_tone );
 	}
+
 	inline volume getVolume( void ) const
 	{
 		return( m_volume );
 	}
+
 	inline panning getPanning( void ) const
 	{
 		return( m_panning );
 	}
 
-	virtual void FASTCALL saveSettings( QDomDocument & _doc,
-							QDomElement & _parent );
-	virtual void FASTCALL loadSettings( const QDomElement & _this );
-	inline virtual QString nodeName( void ) const
+	static QString classNodeName( void )
 	{
 		return( "note" );
 	}
 
+	inline virtual QString nodeName( void ) const
+	{
+		return( classNodeName() );
+	}
+
+	static midiTime FASTCALL quantized( const midiTime & _m,
+							const int _q_grid );
+
+	detuningHelper * detuning( void ) const
+	{
+		return( m_detuning );
+	}
+
+	void editDetuningPattern( void );
+
+	bool hasDetuningInfo( void );
+
+
+protected:
+	virtual void FASTCALL saveSettings( QDomDocument & _doc,
+							QDomElement & _parent );
+	virtual void FASTCALL loadSettings( const QDomElement & _this );
+
+	virtual void undoStep( journalEntry & _je );
+	virtual void redoStep( journalEntry & _je );
+
 
 private:
+	enum actions
+	{
+		CHANGE_KEY, CHANGE_VOLUME, CHANGE_PANNING,
+		CHANGE_LENGTH, CHANGE_POSITION
+	} ;
+
+	static const float MAX_DETUNING;
+
+
 	tones m_tone;
 	octaves m_octave;
 	volume m_volume;
 	panning m_panning;
 	midiTime m_length;
 	midiTime m_pos;
+	detuningHelper * m_detuning;
+
+	void createDetuning( void );
 
 } ;
 
