@@ -1,7 +1,7 @@
 /*
  * plugin.h - class plugin, the base-class and generic interface for all plugins
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -46,9 +46,8 @@
 #endif
 
 
-#include "types.h"
 #include "journalling_object.h"
-#include "embed.h"
+#include "base64.h"
 
 
 #define STRINGIFY_PLUGIN_NAME(s) STR(s)
@@ -56,6 +55,7 @@
 
 
 class QPixmap;
+class QWidget;
 
 
 class plugin : public journallingObject
@@ -63,11 +63,15 @@ class plugin : public journallingObject
 public:
 	enum pluginTypes
 	{
-		INSTRUMENT,	// instrument being used in channel-track
-		EFFECT,		// effect-plugin for effect-board
-		IMPORT_FILTER,	// filter for importing a file
-		EXPORT_FILTER,	// filter for exporting a file
-		UNDEFINED = 255
+		Instrument,	// instrument being used in channel-track
+		Effect,		// effect-plugin for effect-board
+		ImportFilter,	// filter for importing a file
+		ExportFilter,	// filter for exporting a file
+		Tool,		// additional tool (level-meter etc)
+		Library,	// simple library holding a code-base for
+				// several other plugins (e.g. LADSPA-support)
+		Other,
+		Undefined = 255
 	} ;
 
 	// descriptor holds information about a plugin - every external plugin
@@ -82,14 +86,81 @@ public:
 		int version;
 		pluginTypes type;
 		const QPixmap * logo;
+		class subPluginFeatures
+		{
+		public:
+			struct key
+			{
+				inline key( plugin::descriptor * _desc = NULL,
+					const QString & _name = QString::null,
+					const QVariant & _user = QVariant() )
+					:
+					desc( _desc ),
+					name( _name ),
+					user( _user )
+				{
+				}
+
+				inline key( const QString & _dump_data )	
+					:
+					desc( NULL )
+				{
+					const vlist<QVariant> l =
+						base64::decode( _dump_data ).
+								toList();
+					name = l[0].toString();
+					user = l[1];
+				}
+				inline QString dumpBase64( void ) const
+				{
+					return( base64::encode(
+						vlist<QVariant>()
+							<< name << user ) );
+				}
+				plugin::descriptor * desc;
+				QString name;
+				QVariant user;
+			};
+			typedef vlist<key> keyList;
+
+			subPluginFeatures( plugin::pluginTypes _type ) :
+				m_type( _type )
+			{
+			}
+
+			virtual ~subPluginFeatures()
+			{
+			}
+
+			virtual void fillDescriptionWidget( QWidget *,
+								const key * )
+			{
+			}
+			virtual void listSubPluginKeys( plugin::descriptor *,
+							keyList & )
+			{
+			}
+
+			virtual const QStringList & supportedExtensions( void )
+			{
+				static QStringList no_extensions;
+				return( no_extensions );
+			}
+
+
+		protected:
+			const plugin::pluginTypes m_type;
+		}
+			* sub_plugin_features;
+
 	} ;
 
 	// contructor of a plugin
-	plugin( const descriptor * _descriptor, engine * _engine );
+	plugin( const descriptor * _descriptor );
 	virtual ~plugin();
 
 	// returns public-name out of descriptor
-	inline QString publicName( void ) const
+	virtual inline QString publicName( void ) const
 	{
 		return( m_descriptor->public_name );
 	}
@@ -121,7 +192,21 @@ public:
 	static plugin * FASTCALL instantiate( const QString & _plugin_name,
 							void * _data );
 
-	// fills given vector with descriptors for all available plugins
+	// some plugins run external programs for doing their actual work
+	// (e.g. LVSL-server) or can run in separate worker-threads, so the
+	// mixer can schedule processing for parallelizing work which is very
+	// important for at least trying to use the full power of SMP-systems,
+	// otherwise the mixer will create according threads on it's own which
+	// of course isn't that efficient
+	virtual bool supportsParallelizing( void ) const;
+
+	// plugins supporting parallelizing, should re-implement that as the
+	// mixer will call this at the end of processing according chain
+	// of plugins
+	virtual void waitForWorkerThread( void );
+
+
+	// fills given vector with descriptors of all available plugins
 	static void FASTCALL getDescriptorsOfAvailPlugins(
 					vvector<descriptor> & _plugin_descs );
 
