@@ -1,7 +1,7 @@
 /*
  * visualization_widget.h - widget for visualization of sound-data
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -47,7 +47,7 @@
 class QTimer;
 
 
-class visualizationWidget : public QWidget, public engineObject
+class visualizationWidget : public QWidget
 {
 	Q_OBJECT
 public:
@@ -56,7 +56,6 @@ public:
 		SIMPLE		// add more here
 	} ;
 	visualizationWidget( const QPixmap & _bg, QWidget * _parent,
-					engine * _engine,
 					visualizationTypes _vtype = SIMPLE );
 	virtual ~visualizationWidget();
 
