@@ -1,7 +1,7 @@
 /*
  * audio_file_processor.cpp - instrument for using audio-files
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -46,21 +46,23 @@
 
 
 #include "audio_file_processor.h"
+#include "engine.h"
 #include "song_editor.h"
 #include "instrument_track.h"
 #include "note_play_handle.h"
-#include "paths.h"
 #include "interpolation.h"
-#include "buffer_allocator.h"
+#include "file_browser.h"
+#include "gui_templates.h"
 #include "pixmap_button.h"
 #include "knob.h"
 #include "tooltip.h"
 #include "string_pair_drag.h"
 #include "mmp.h"
+#include "volume_knob.h"
+
 
 #undef SINGLE_SOURCE_COMPILE
 #include "embed.cpp"
-#include "volume_knob.h"
 
 
 extern "C"
@@ -76,8 +78,9 @@ plugin::descriptor audiofileprocessor_plugin_descriptor =
 				"instrument-track" ),
 	"Tobias Doerffel <tobydox/at/users.sf.net>",
 	0x0100,
-	plugin::INSTRUMENT,
-	new QPixmap( PLUGIN_NAME::getIconPixmap( "logo" ) )
+	plugin::Instrument,
+	new QPixmap( PLUGIN_NAME::getIconPixmap( "logo" ) ),
+	new audioFileProcessor::subPluginFeatures( plugin::Instrument )
 } ;
 
 }
@@ -90,7 +93,6 @@ QPixmap * audioFileProcessor::s_artwork = NULL;
 audioFileProcessor::audioFileProcessor( instrumentTrack * _channel_track ) :
 	instrument( _channel_track, &audiofileprocessor_plugin_descriptor ),
 	specialBgHandlingWidget( PLUGIN_NAME::getIconPixmap( "artwork" ) ),
-	m_sampleBuffer( eng(), "" ),
 	m_drawMethod( sampleBuffer::LINE_CONNECT )
 {
 	connect( &m_sampleBuffer, SIGNAL( sampleUpdated() ), this,
@@ -103,7 +105,7 @@ audioFileProcessor::audioFileProcessor( instrumentTrack * _channel_track ) :
 	}
 
 
-	m_openAudioFileButton = new pixmapButton( this, NULL, eng(), NULL );
+	m_openAudioFileButton = new pixmapButton( this, NULL, NULL );
 	m_openAudioFileButton->setCursor( QCursor( Qt::PointingHandCursor ) );
 	m_openAudioFileButton->move( 200, 90 );
 	m_openAudioFileButton->setActiveGraphic( embed::getIconPixmap(
@@ -128,7 +130,7 @@ audioFileProcessor::audioFileProcessor( instrumentTrack * _channel_track ) :
 			"are not reset, so please don't wonder if your sample "
 			"doesn't sound like the original one..." ) );
 	
-	m_reverseButton = new pixmapButton( this, NULL, eng(), NULL );
+	m_reverseButton = new pixmapButton( this, NULL, NULL );
 	m_reverseButton->setCheckable( TRUE );
 	m_reverseButton->move( 160, 124 );
 	m_reverseButton->setActiveGraphic( PLUGIN_NAME::getIconPixmap(
@@ -148,7 +150,7 @@ audioFileProcessor::audioFileProcessor( instrumentTrack * _channel_track ) :
 			"This is useful for cool effects, e.g. a reversed "
 			"crash." ) );
 
-	m_loopButton = new pixmapButton( this, NULL, eng(), NULL );
+	m_loopButton = new pixmapButton( this, tr( "Loop" ), _channel_track );
 	m_loopButton->setCheckable( TRUE );
 	m_loopButton->move( 180, 124 );
 	m_loopButton->setActiveGraphic( PLUGIN_NAME::getIconPixmap(
@@ -169,8 +171,7 @@ audioFileProcessor::audioFileProcessor( instrumentTrack * _channel_track ) :
 			"This is useful for things like string- and choir-"
 			"samples." ) );
 
-	m_ampKnob = new volumeKnob( knobDark_28, this, tr( "Amplify" ), eng(),
-									NULL );
+	m_ampKnob = new volumeKnob( knobDark_28, this, tr( "Amplify" ), NULL );
 	m_ampKnob->setRange( 0, 500, 1.0f );
 	m_ampKnob->move( 6, 114 );
 	m_ampKnob->setInitValue( 100.0f );
@@ -189,7 +190,7 @@ audioFileProcessor::audioFileProcessor( instrumentTrack * _channel_track ) :
 			"actual sample-file isn't touched!)" ) );
 
 	m_startKnob = new knob( knobDark_28, this, tr( "Start of sample" ),
-								eng(), NULL );
+							_channel_track );
 	m_startKnob->setRange( 0.0f, 1.0f, 0.00001f );
 	m_startKnob->move( 46, 114 );
 	m_startKnob->setInitValue( 0.0f );
@@ -208,8 +209,8 @@ audioFileProcessor::audioFileProcessor( instrumentTrack * _channel_track ) :
 			"which AudioFileProcessor returns if a note is longer "
 			"than the sample between start- and end-point." ) );
 
-	m_endKnob = new knob( knobDark_28, this, tr( "End of sample" ), eng(),
-									NULL );
+	m_endKnob = new knob( knobDark_28, this, tr( "End of sample" ),
+							_channel_track );
 	m_endKnob->setRange( 0.0f, 1.0f, 0.00001f );
 	m_endKnob->move( 84, 114 );
 	m_endKnob->setInitValue( 1.0f );
@@ -228,7 +229,7 @@ audioFileProcessor::audioFileProcessor( instrumentTrack * _channel_track ) :
 			"AudioFileProcessor returns if a note is longer than "
 			"the sample between start- and end-point." ) );
 
-	m_viewLinesPB = new pixmapButton( this, NULL, eng(), NULL );
+	m_viewLinesPB = new pixmapButton( this, NULL, NULL );
 	m_viewLinesPB->move( 154, 158 );
 	m_viewLinesPB->setBgGraphic( getBackground( m_viewLinesPB ) );
 	if( m_drawMethod == sampleBuffer::LINE_CONNECT )
@@ -247,7 +248,7 @@ audioFileProcessor::audioFileProcessor( instrumentTrack * _channel_track ) :
 			"sound itself. It just gives you another view to your "
 			"sample." ) );
 
-	m_viewDotsPB = new pixmapButton( this, NULL, eng(), NULL );
+	m_viewDotsPB = new pixmapButton( this, NULL, NULL );
 	m_viewDotsPB->move( 204, 158 );
 	m_viewDotsPB->setBgGraphic( getBackground( m_viewDotsPB ) );
 	if( m_drawMethod == sampleBuffer::DOTS )
@@ -266,7 +267,7 @@ audioFileProcessor::audioFileProcessor( instrumentTrack * _channel_track ) :
 			"It just gives you another view to your sample." ) );
 	
 	automatableButtonGroup * view_group = new automatableButtonGroup( this,
-							NULL, eng(), NULL );
+								NULL, NULL );
 	view_group->addButton( m_viewLinesPB );
 	view_group->addButton( m_viewDotsPB );
 
@@ -295,15 +296,11 @@ void audioFileProcessor::saveSettings( QDomDocument & _doc,
 		QString s;
 		_this.setAttribute( "sampledata", m_sampleBuffer.toBase64( s ) );
 	}
-	_this.setAttribute( "sframe", QString::number(
-						m_sampleBuffer.startFrame() /
-					(float)m_sampleBuffer.frames() ) );
-	_this.setAttribute( "eframe", QString::number(
-						m_sampleBuffer.endFrame() /
-					(float)m_sampleBuffer.frames() ) );
 	m_reverseButton->saveSettings( _doc, _this, "reversed" );
 	m_loopButton->saveSettings( _doc, _this, "looped" );
 	m_ampKnob->saveSettings( _doc, _this, "amp" );
+	m_startKnob->saveSettings( _doc, _this, "sframe" );
+	m_endKnob->saveSettings( _doc, _this, "eframe" );
 }
 
 
@@ -319,11 +316,14 @@ void audioFileProcessor::loadSettings( const QDomElement & _this )
 	{
 		m_sampleBuffer.loadFromBase64( _this.attribute( "srcdata" ) );
 	}
-	setStartAndEndKnob( _this.attribute( "sframe" ).toFloat(),
-				_this.attribute( "eframe" ).toFloat() );  
 	m_reverseButton->loadSettings( _this, "reversed" );
 	m_loopButton->loadSettings( _this, "looped" );
 	m_ampKnob->loadSettings( _this, "amp" );
+	m_startKnob->loadSettings( _this, "sframe" );
+	m_endKnob->loadSettings( _this, "eframe" );
+
+	startKnobChanged( m_startKnob->value()  );
+	endKnobChanged( m_endKnob->value() );
 }
 
 
@@ -355,10 +355,8 @@ QString audioFileProcessor::nodeName( void ) const
 
 Uint32 audioFileProcessor::getBeatLen( notePlayHandle * _n ) const
 {
-	const float freq_factor = BASE_FREQ /
-				( getInstrumentTrack()->frequency( _n ) *
-						DEFAULT_SAMPLE_RATE /
-					eng()->getMixer()->sampleRate() );
+	const float freq_factor = BASE_FREQ / _n->frequency() *
+			engine::getMixer()->sampleRate() / DEFAULT_SAMPLE_RATE;
 
 	return( static_cast<Uint32>( floorf( ( m_sampleBuffer.endFrame() -
 						m_sampleBuffer.startFrame() ) *
@@ -383,30 +381,32 @@ void audioFileProcessor::setAudioFile( const QString & _audio_file, bool _rename
 	// else we don't touch the channel-name, because the user named it self
 
 	m_sampleBuffer.setAudioFile( _audio_file );
-	setStartAndEndKnob( 0.0f, 1.0f );
+	startKnobChanged( m_startKnob->value()  );
+	endKnobChanged( m_endKnob->value() );
 }
 
 
 
 
 
-void audioFileProcessor::playNote( notePlayHandle * _n )
+void audioFileProcessor::playNote( notePlayHandle * _n, bool )
 {
-	const Uint32 frames = eng()->getMixer()->framesPerAudioBuffer();
-	sampleFrame * buf = bufferAllocator::alloc<sampleFrame>( frames );
+	const fpp_t frames = _n->framesLeftForCurrentPeriod();
+	sampleFrame * buf = new sampleFrame[frames];
 
-	// calculate frequency of note
-	const float note_freq = getInstrumentTrack()->frequency( _n ) /
-					( eng()->getMixer()->sampleRate() /
-							DEFAULT_SAMPLE_RATE );
-	if( m_sampleBuffer.play( buf, _n->totalFramesPlayed(),
-					frames, note_freq,
-					m_loopButton->isChecked(),
-					&_n->m_pluginData ) == TRUE )
+	if( !_n->m_pluginData )
 	{
+		_n->m_pluginData = new handleState( _n->hasDetuningInfo() );
+	}
+
+	if( m_sampleBuffer.play( buf, (handleState *)_n->m_pluginData,
+					frames, _n->frequency(),
+					m_loopButton->isChecked() ) == TRUE )
+	{
+		applyRelease( buf, _n );
 		getInstrumentTrack()->processAudioBuffer( buf, frames, _n );
 	}
-	bufferAllocator::free( buf );
+	delete[] buf;
 }
 
 
@@ -414,10 +414,7 @@ void audioFileProcessor::playNote( notePlayHandle * _n )
 
 void audioFileProcessor::deleteNotePluginData( notePlayHandle * _n )
 {
-	if( _n->m_pluginData != NULL )
-	{
-		m_sampleBuffer.deleteResamplingData( &_n->m_pluginData );
-	}
+	delete (handleState *)_n->m_pluginData;
 }
 
 
@@ -425,12 +422,64 @@ void audioFileProcessor::deleteNotePluginData( notePlayHandle * _n )
 
 void audioFileProcessor::dragEnterEvent( QDragEnterEvent * _dee )
 {
-	if( stringPairDrag::processDragEnterEvent( _dee,
-		QString( "samplefile,tco_%1" ).arg( track::SAMPLE_TRACK ) ) ==
-									FALSE )
+#ifdef QT4
+	if( _dee->mimeData()->hasFormat( stringPairDrag::mimeType() ) )
+	{
+		QString txt = _dee->mimeData()->data(
+						stringPairDrag::mimeType() );
+		if( txt.section( ':', 0, 0 ) == QString( "tco_%1" ).arg(
+							track::SAMPLE_TRACK ) )
+		{
+			_dee->acceptProposedAction();
+		}
+		else if( txt.section( ':', 0, 0 ) == "samplefile" )
+		{
+			_dee->acceptProposedAction();
+		}
+		else
+		{
+			_dee->ignore();
+		}
+	}
+	else
 	{
 		_dee->ignore();
 	}
+#else
+	QString txt = _dee->encodedData( stringPairDrag::mimeType() );
+	if( txt != "" )
+	{
+		if( txt.section( ':', 0, 0 ) == QString( "tco_%1" ).arg(
+							track::SAMPLE_TRACK ) )
+		{
+			_dee->accept();
+			return;
+		}
+		if( txt.section( ':', 0, 0 ) == "samplefile"
+			&& subPluginFeatures::supported_extensions().contains(
+				fileItem::extension( txt.section( ':', 1 ) ) ) )
+		{
+			_dee->accept();
+			return;
+		}
+		_dee->ignore();
+		return;
+	}
+
+	txt = QString( _dee->encodedData( "text/plain" ) );
+	if( txt != "" )
+	{
+		QString file = QUriDrag::uriToLocalFile(
+							txt.stripWhiteSpace() );
+		if( file && subPluginFeatures::supported_extensions().contains(
+						fileItem::extension( file ) ) )
+		{
+			_dee->accept();
+			return;
+		}
+	}
+	_dee->ignore();
+#endif
 }
 
 
@@ -444,6 +493,7 @@ void audioFileProcessor::dropEvent( QDropEvent * _de )
 	{
 		setAudioFile( value );
 		_de->accept();
+		return;
 	}
 	else if( type == QString( "tco_%1" ).arg( track::SAMPLE_TRACK ) )
 	{
@@ -451,11 +501,21 @@ void audioFileProcessor::dropEvent( QDropEvent * _de )
 		setAudioFile( mmp.content().firstChild().toElement().
 							attribute( "src" ) );
 		_de->accept();
+		return;
 	}
-	else
+
+#ifndef QT4
+	QString txt = _de->encodedData( "text/plain" );
+	if( txt != "" )
 	{
-		_de->ignore();
+		setAudioFile( QUriDrag::uriToLocalFile(
+						txt.stripWhiteSpace() ) );
+		_de->accept();
+		return;
 	}
+#endif
+
+	_de->ignore();
 }
 
 
@@ -478,18 +538,18 @@ void audioFileProcessor::paintEvent( QPaintEvent * )
  	QString file_name = "";
 	Uint16 idx = m_sampleBuffer.audioFile().length();
 
-	p.setFont( pointSize<8>( p.font() ) );
+	p.setFont( pointSize<8>( font() ) );
 
-	QFontMetrics fm( font() );
+	QFontMetrics fm( p.font() );
 
 	// simple algorithm for creating a text from the filename that
 	// matches in the white rectangle
 #ifdef QT4
 	while( idx > 0 &&
-		fm.size( Qt::TextSingleLine, file_name + "..." ).width() < 225 )
+		fm.size( Qt::TextSingleLine, file_name + "..." ).width() < 210 )
 #else
 	while( idx > 0 &&
-		fm.size( Qt::SingleLine, file_name + "..." ).width() < 225 )
+		fm.size( Qt::SingleLine, file_name + "..." ).width() < 210 )
 #endif
 	{
 		file_name = m_sampleBuffer.audioFile()[--idx] + file_name;
@@ -554,7 +614,7 @@ void audioFileProcessor::sampleUpdated( void )
 void audioFileProcessor::reverseBtnToggled( bool _on )
 {
 	m_sampleBuffer.setReversed( _on );
-	eng()->getSongEditor()->setModified();
+	engine::getSongEditor()->setModified();
 }
 
 
@@ -592,27 +652,6 @@ void audioFileProcessor::ampKnobChanged( float _val )
 
 
 
-void audioFileProcessor::setStartAndEndKnob( float _s, float _e )
-{
-/*	// because the signal-handlers of valuechanges of start- and end-knob
-	// do range checking, depending on value of the other knob, we have to
-	// disconnect the signal-handlers, set then the values, connect again
-	// and then let the changes take effect...
-	m_startKnob->disconnect();
-	m_endKnob->disconnect();*/
-	m_startKnob->setValue( _s );
-	m_endKnob->setValue( _e );
-/*	connect( m_startKnob, SIGNAL( valueChanged( float ) ), this,
-					SLOT( startKnobChanged( float ) ) );
-	connect( m_endKnob, SIGNAL( valueChanged( float ) ), this,
-					SLOT( endKnobChanged( float ) ) );*/
-	startKnobChanged( _s );
-	endKnobChanged( _e );
-}
-
-
-
-
 void audioFileProcessor::startKnobChanged( float _new_value )
 {
 	if( _new_value < m_endKnob->value() )
@@ -661,13 +700,38 @@ void audioFileProcessor::openAudioFile( void )
 	if( af != "" )
 	{
 		setAudioFile( af );
-		eng()->getSongEditor()->setModified();
+		engine::getSongEditor()->setModified();
 	}
 }
 
 
 
 
+
+
+
+
+audioFileProcessor::subPluginFeatures::subPluginFeatures(
+						plugin::pluginTypes _type ) :
+	plugin::descriptor::subPluginFeatures( _type )
+{
+}
+
+
+
+
+const QStringList & audioFileProcessor::subPluginFeatures::supported_extensions(
+									void )
+{
+	static QStringList extensions = QStringList()
+				<< "wav" << "ogg" << "spx" << "au" << "voc"
+				<< "aif" << "aiff" << "flac" << "raw";
+	return( extensions );
+}
+
+
+
+
 extern "C"
 {
 
