@@ -2,7 +2,7 @@
  * audio_file_processor.h - declaration of class audioFileProcessor
  *                          (instrument-plugin for using audio-files)
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -53,10 +53,26 @@ class audioFileProcessor : public instrument, public specialBgHandlingWidget
 {
 	Q_OBJECT
 public:
+	class subPluginFeatures : public plugin::descriptor::subPluginFeatures
+	{
+	public:
+		subPluginFeatures( plugin::pluginTypes _type );
+
+		virtual const QStringList & supportedExtensions( void )
+		{
+			return( supported_extensions() );
+		}
+
+		static const QStringList & supported_extensions( void );
+
+	} ;
+
+
 	audioFileProcessor( instrumentTrack * _channel_track );
 	virtual ~audioFileProcessor();
 
-	virtual void FASTCALL playNote( notePlayHandle * _n );
+	virtual void FASTCALL playNote( notePlayHandle * _n,
+						bool _try_parallelizing );
 	virtual void FASTCALL deleteNotePluginData( notePlayHandle * _n );
 
 	virtual void FASTCALL saveSettings( QDomDocument & _doc,
@@ -70,6 +86,11 @@ public:
 
 	virtual Uint32 FASTCALL getBeatLen( notePlayHandle * _n ) const;
 
+	virtual f_cnt_t desiredReleaseFrames( void ) const
+	{
+		return( 128 );
+	}
+
 
 public slots:
 	void setAudioFile( const QString & _audio_file, bool _rename = TRUE );
@@ -93,6 +114,8 @@ protected:
 
 
 private:
+	typedef sampleBuffer::handleState handleState;
+
 	static QPixmap * s_artwork;
 
 
@@ -112,7 +135,6 @@ private:
 
 
 	void updateSample( void );
-	void FASTCALL setStartAndEndKnob( float _s, float _e );
 
 } ;
 
