@@ -1,7 +1,7 @@
 /*
  * vestige.cpp - instrument-plugin for hosting VST-plugins
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,12 +17,14 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
 
+#include "vestige.h"
+
 #include "qt3support.h"
 
 #ifdef QT4
@@ -49,18 +51,18 @@
 #endif
 
 
-#include "instrument_track.h"
-#include "note_play_handle.h"
-#include "buffer_allocator.h"
-#include "mixer.h"
+#include "engine.h"
+#include "gui_templates.h"
 #include "instrument_play_handle.h"
-#include "pixmap_button.h"
-#include "tooltip.h"
-#include "spc_bg_hndl_widget.h"
-#include "vestige.h"
-#include "text_float.h"
-#include "song_editor.h"
+#include "instrument_track.h"
 #include "lvsl_client.h"
+#include "mixer.h"
+#include "note_play_handle.h"
+#include "pixmap_button.h"
+#include "song_editor.h"
+#include "spc_bg_hndl_widget.h"
+#include "text_float.h"
+#include "tooltip.h"
 
 #undef SINGLE_SOURCE_COMPILE
 #include "embed.cpp"
@@ -77,8 +79,9 @@ plugin::descriptor vestige_plugin_descriptor =
 			"VST-host for using VST(i)-plugins within LMMS" ),
 	"Tobias Doerffel <tobydox/at/users.sf.net>",
 	0x0100,
-	plugin::INSTRUMENT,
-	new QPixmap( PLUGIN_NAME::getIconPixmap( "logo" ) )
+	plugin::Instrument,
+	new QPixmap( PLUGIN_NAME::getIconPixmap( "logo" ) ),
+	NULL
 } ;
 
 }
@@ -87,8 +90,8 @@ plugin::descriptor vestige_plugin_descriptor =
 QPixmap * vestigeInstrument::s_artwork = NULL;
 
 
-vestigeInstrument::vestigeInstrument( instrumentTrack * _channel_track ) :
-	instrument( _channel_track, &vestige_plugin_descriptor ),
+vestigeInstrument::vestigeInstrument( instrumentTrack * _instrument_track ) :
+	instrument( _instrument_track, &vestige_plugin_descriptor ),
 	specialBgHandlingWidget( PLUGIN_NAME::getIconPixmap( "artwork" ) ),
 	m_plugin( NULL ),
 	m_pluginMutex()
@@ -107,10 +110,7 @@ vestigeInstrument::vestigeInstrument( instrumentTrack * _channel_track ) :
 	setErasePixmap( *s_artwork );
 #endif
 
-	connect( eng()->getSongEditor(), SIGNAL( tempoChanged( bpm_t ) ),
-					 this, SLOT( changeTempo( bpm_t ) ) );
-
-	m_openPluginButton = new pixmapButton( this, NULL, eng(), NULL );
+	m_openPluginButton = new pixmapButton( this, NULL, NULL );
 	m_openPluginButton->setCheckable( FALSE );
 	m_openPluginButton->setCursor( Qt::PointingHandCursor );
 	m_openPluginButton->move( 200, 70 );
@@ -155,9 +155,14 @@ vestigeInstrument::vestigeInstrument( instrumentTrack * _channel_track ) :
 	connect( note_off_all_btn, SIGNAL( clicked() ), this,
 							SLOT( noteOffAll() ) );
 
+	for( int i = 0; i < NOTES; ++i )
+	{
+		m_noteStates[i] = OFF;
+	}
+
 	// now we need a play-handle which cares for calling play()
 	instrumentPlayHandle * iph = new instrumentPlayHandle( this );
-	eng()->getMixer()->addPlayHandle( iph );
+	engine::getMixer()->addPlayHandle( iph );
 }
 
 
@@ -165,9 +170,7 @@ vestigeInstrument::vestigeInstrument( instrumentTrack * _channel_track ) :
 
 vestigeInstrument::~vestigeInstrument()
 {
-	// this single call automates the rest of cleanup like trashing our
-	// play-handle and so on
-	invalidate();
+	engine::getMixer()->removePlayHandles( getInstrumentTrack() );
 	closePlugin();
 }
 
@@ -180,30 +183,7 @@ void vestigeInstrument::loadSettings( const QDomElement & _this )
 	m_pluginMutex.lock();
 	if( m_plugin != NULL )
 	{
-		if( m_plugin->pluginWidget() != NULL )
-		{
-			if( _this.attribute( "guivisible" ).toInt() )
-			{
-				m_plugin->pluginWidget()->show();
-			}
-			else
-			{
-				m_plugin->pluginWidget()->hide();
-			}
-		}
-		const Sint32 num_params = _this.attribute(
-							"numparams" ).toInt();
-		if( num_params > 0 )
-		{
-			QMap<QString, QString> dump;
-			for( Sint32 i = 0; i < num_params; ++i )
-			{
-				const QString key = "param" +
-							QString::number( i );
-				dump[key] = _this.attribute( key );
-			}
-			m_plugin->setParameterDump( dump );
-		}
+		m_plugin->loadSettings( _this );
 	}
 	m_pluginMutex.unlock();
 }
@@ -217,22 +197,7 @@ void vestigeInstrument::saveSettings( QDomDocument & _doc, QDomElement & _this )
 	m_pluginMutex.lock();
 	if( m_plugin != NULL )
 	{
-		if( m_plugin->pluginWidget() != NULL )
-		{
-			_this.setAttribute( "guivisible",
-					m_plugin->pluginWidget()->isVisible() );
-		}
-		const QMap<QString, QString> & dump = m_plugin->parameterDump();
-		_this.setAttribute( "numparams", dump.size() );
-		for( QMap<QString, QString>::const_iterator it = dump.begin();
-							it != dump.end(); ++it )
-		{
-#ifdef QT4
-			_this.setAttribute( it.key(), it.value() );
-#else
-			_this.setAttribute( it.key(), it.data() );
-#endif
-		}
+		m_plugin->saveSettings( _doc, _this );
 	}
 	m_pluginMutex.unlock();
 }
@@ -257,7 +222,7 @@ void vestigeInstrument::setParameter( const QString & _param,
 		const bool set_ch_name = ( m_plugin != NULL &&
 			getInstrumentTrack()->name() == m_plugin->name() ) ||
 				getInstrumentTrack()->name() ==
-						instrumentTrack::tr( "Default" );
+					instrumentTrack::tr( "Default" );
 		m_pluginMutex.unlock();
 
 		closePlugin();
@@ -269,7 +234,7 @@ void vestigeInstrument::setParameter( const QString & _param,
 				PLUGIN_NAME::getIconPixmap( "logo", 24, 24 ),
 									0 );
 		m_pluginMutex.lock();
-		m_plugin = new remoteVSTPlugin( m_pluginDLL, eng() );
+		m_plugin = new remoteVSTPlugin( m_pluginDLL );
 		if( m_plugin->failed() )
 		{
 			m_pluginMutex.unlock();
@@ -299,7 +264,12 @@ void vestigeInstrument::setParameter( const QString & _param,
 			return;
 		}*/
 		m_plugin->showEditor();
-		m_plugin->setTempo( eng()->getSongEditor()->getTempo() );
+		connect( engine::getSongEditor(),
+					SIGNAL( tempoChanged( bpm_t ) ),
+				m_plugin, SLOT( setTempo( bpm_t ) ) );
+		m_plugin->setTempo( engine::getSongEditor()->getTempo() );
+		connect( engine::getMixer(), SIGNAL( sampleRateChanged() ),
+					m_plugin, SLOT( updateSampleRate() ) );
 		if( set_ch_name == TRUE )
 		{
 			getInstrumentTrack()->setName( m_plugin->name() );
@@ -323,22 +293,47 @@ void vestigeInstrument::setParameter( const QString & _param,
 
 
 
-void vestigeInstrument::play( void )
+void vestigeInstrument::waitForWorkerThread( void )
 {
-	QMutexLocker ml( &m_pluginMutex );
+	m_pluginMutex.lock();
 	if( m_plugin == NULL )
 	{
+		m_pluginMutex.unlock();
 		return;
 	}
 
-	const fpab_t frames = eng()->getMixer()->framesPerAudioBuffer();
-	sampleFrame * buf = bufferAllocator::alloc<sampleFrame>( frames );
+	const fpp_t frames = engine::getMixer()->framesPerPeriod();
+	sampleFrame * buf = new sampleFrame[frames];
 
-	m_plugin->process( NULL, buf );
-	
-	getInstrumentTrack()->processAudioBuffer( buf, frames, NULL );
+	if( m_plugin->waitForProcessingFinished( buf ) )
+	{
+		getInstrumentTrack()->processAudioBuffer( buf, frames, NULL );
+	}
 
-	bufferAllocator::free( buf );
+	m_pluginMutex.unlock();
+
+	delete[] buf;
+}
+
+
+
+
+void vestigeInstrument::play( bool _try_parallelizing )
+{
+	m_pluginMutex.lock();
+	if( m_plugin == NULL )
+	{
+		m_pluginMutex.unlock();
+		return;
+	}
+
+	m_plugin->process( NULL, NULL, FALSE );
+	m_pluginMutex.unlock();
+
+	if( !_try_parallelizing )
+	{
+		waitForWorkerThread();
+	}
 }
 
 
@@ -346,14 +341,26 @@ void vestigeInstrument::play( void )
 
 
 
-void vestigeInstrument::playNote( notePlayHandle * _n )
+void vestigeInstrument::playNote( notePlayHandle * _n, bool )
 {
 	m_pluginMutex.lock();
 	if( _n->totalFramesPlayed() == 0 && m_plugin != NULL )
 	{
-		m_plugin->enqueueMidiEvent( midiEvent( NOTE_ON, 0,
-					getInstrumentTrack()->masterKey( _n ),
-					_n->getVolume() ), _n->framesAhead() );
+		const int k = getInstrumentTrack()->masterKey( _n );
+		if( m_noteStates[k] > OFF )
+		{
+			m_plugin->enqueueMidiEvent( midiEvent( NOTE_OFF, 0,
+								k, 0 ), 0 );
+			m_noteStates[k] = IGNORE_NEXT_NOTEOFF;
+		}
+		else
+		{
+			m_noteStates[k] = ON;
+		}
+		m_plugin->enqueueMidiEvent( midiEvent( NOTE_ON, 0, k,
+					_n->getVolume() ), _n->offset() );
+		// notify when the handle stops, call to deleteNotePluginData
+		_n->m_pluginData = _n;
 	}
 	m_pluginMutex.unlock();
 }
@@ -366,9 +373,17 @@ void vestigeInstrument::deleteNotePluginData( notePlayHandle * _n )
 	m_pluginMutex.lock();
 	if( m_plugin != NULL )
 	{
-		m_plugin->enqueueMidiEvent( midiEvent( NOTE_OFF, 0,
-					getInstrumentTrack()->masterKey( _n ),
+		const int k = getInstrumentTrack()->masterKey( _n );
+		if( m_noteStates[k] == ON )
+		{
+			m_plugin->enqueueMidiEvent( midiEvent( NOTE_OFF, 0, k,
 								0 ), 0 );
+			m_noteStates[k] = OFF;
+		}
+		else if( m_noteStates[k] == IGNORE_NEXT_NOTEOFF )
+		{
+			m_noteStates[k] = ON;
+		}
 	}
 	m_pluginMutex.unlock();
 }
@@ -376,6 +391,21 @@ void vestigeInstrument::deleteNotePluginData( notePlayHandle * _n )
 
 
 
+bool vestigeInstrument::handleMidiEvent( const midiEvent & _me,
+						const midiTime & _time )
+{
+	m_pluginMutex.lock();
+	if( m_plugin != NULL )
+	{
+		m_plugin->enqueueMidiEvent( _me, _time );
+	}
+	m_pluginMutex.unlock();
+	return( TRUE );
+}
+
+
+
+
 void vestigeInstrument::openPlugin( void )
 {
 #ifdef QT4
@@ -474,7 +504,7 @@ void vestigeInstrument::noteOffAll( void )
 
 
 
-
+/*
 void vestigeInstrument::changeTempo( bpm_t _new_tempo )
 {
 	m_pluginMutex.lock();
@@ -484,7 +514,7 @@ void vestigeInstrument::changeTempo( bpm_t _new_tempo )
 	}
 	m_pluginMutex.unlock();
 }
-
+*/
 
 
 
