@@ -4,7 +4,7 @@
  * note_play_handle.cpp - implementation of class notePlayHandle, part of
  *                        play-engine
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -20,66 +20,97 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
 
 #include "note_play_handle.h"
-#include "instrument_track.h"
+#include "automatable_object_templates.h"
+#include "config_mgr.h"
+#include "detuning_helper.h"
 #include "envelope_tab_widget.h"
-#include "midi.h"
+#include "instrument_track.h"
 #include "midi_port.h"
 #include "song_editor.h"
-#include "piano_widget.h"
-#include "config_mgr.h"
-#include "project_journal.h"
+
+
+inline notePlayHandle::baseDetuning::baseDetuning(
+						detuningHelper * _detuning ) :
+	m_detuning( _detuning )
+{
+	m_level = m_detuning->getAutomationPattern()->valueAt( 0 );
+	m_value = m_detuning->value( m_level );
+}
+
+
+
+
+inline void notePlayHandle::baseDetuning::setLevel( int _level )
+{
+	m_level = _level;
+	m_value = m_detuning->value( m_level );
+}
+
+
+
+
+
+
 
 
 notePlayHandle::notePlayHandle( instrumentTrack * _it,
-						const f_cnt_t _frames_ahead,
+						const f_cnt_t _offset,
 						const f_cnt_t _frames,
 						const note & _n,
+						notePlayHandle * _parent,
 						const bool _arp_note ) :
-	playHandle( NOTE_PLAY_HANDLE ),
-	note( NULL, _n.length(), _n.pos(), _n.tone(), _n.octave(),
-					_n.getVolume(), _n.getPanning() ),
+	playHandle( NotePlayHandle, _offset ),
+	note( _n.length(), _n.pos(), _n.tone(), _n.octave(),
+			_n.getVolume(), _n.getPanning(), _n.detuning() ),
 	m_pluginData( NULL ),
 	m_filter( NULL ),
 	m_instrumentTrack( _it ),
 	m_frames( 0 ),
-	m_framesAhead( _frames_ahead ),
 	m_totalFramesPlayed( 0 ), 
 	m_framesBeforeRelease( 0 ),
 	m_releaseFramesToDo( 0 ),
 	m_releaseFramesDone( 0 ),
 	m_released( FALSE ),
-	m_baseNote( TRUE  ),
+	m_baseNote( _parent == NULL  ),
 	m_arpNote( _arp_note ),
-	m_muted( FALSE )
+	m_muted( FALSE ),
+	m_bbTrack( NULL ),
+#if SINGERBOT_SUPPORT
+	m_patternIndex( 0 ),
+#endif
+	m_orig_bpm( engine::getSongEditor()->getTempo() )
 {
-	setDetuning( _n.detuning() );
-	if( detuning() )
+	if( m_baseNote )
 	{
-		connect( m_instrumentTrack,
-				SIGNAL( sentMidiTime( const midiTime & ) ),
-				this,
-				SLOT( processMidiTime( const midiTime & ) ) );
-		processMidiTime( pos() );
-		connect( detuning(), SIGNAL( valueChanged( float ) ),
-					this, SLOT( updateFrequency() ) );
+		m_base_detuning = new baseDetuning( detuning() );
+		m_instrumentTrack->m_processHandles.push_back( this );
 	}
-	connect( m_instrumentTrack, SIGNAL( baseNoteChanged() ),
-					this, SLOT( updateFrequency() ) );
+	else
+	{
+		m_base_detuning = _parent->m_base_detuning;
+
+		_parent->m_subNotes.push_back( this );
+		// if there was an arp-note added and parent is a base-note
+		// we set arp-note-flag for indicating that parent is an
+		// arpeggio-base-note
+		_parent->m_arpNote = arpNote() && _parent->baseNote();
+
+		m_bbTrack = _parent->m_bbTrack;
+#if SINGERBOT_SUPPORT
+		m_patternIndex = _parent->m_patternIndex;
+#endif
+	}
+
 	updateFrequency();
 
 	setFrames( _frames );
-	if( !configManager::inst()->value( "ui",
-						"manualchannelpiano" ).toInt() )
-	{
-		m_instrumentTrack->m_pianoWidget->setKeyState( key(), TRUE );
-	}
 	// send MIDI-note-on-event
 	m_instrumentTrack->processOutEvent( midiEvent( NOTE_ON,
 				m_instrumentTrack->m_midiPort->outputChannel(),
@@ -88,8 +119,8 @@ notePlayHandle::notePlayHandle( instrumentTrack * _it,
 				(Uint16) ( ( getVolume() / 100.0f ) *
 				( m_instrumentTrack->getVolume() / 100.0f ) *
 							127 ), 0, 127 ) ),
-			midiTime::fromFrames( m_framesAhead,
-		m_instrumentTrack->eng()->getSongEditor()->framesPerTact() ) );
+			midiTime::fromFrames( offset(),
+						engine::framesPerTact64th() ) );
 }
 
 
@@ -102,7 +133,13 @@ notePlayHandle::~notePlayHandle()
 		noteOff( 0 );
 	}
 
-	if( m_instrumentTrack != NULL )
+	if( m_baseNote )
+	{
+		delete m_base_detuning;
+		m_instrumentTrack->m_processHandles.removeAll( this );
+	}
+
+	if( m_pluginData != NULL )
 	{
 		m_instrumentTrack->deleteNotePluginData( this );
 	}
@@ -120,28 +157,26 @@ notePlayHandle::~notePlayHandle()
 
 
 
-void notePlayHandle::play( void )
+void notePlayHandle::play( bool _try_parallelizing )
 {
-	if( m_muted == TRUE || m_instrumentTrack == NULL )
+	if( m_muted == TRUE )
 	{
 		return;
 	}
 
 	if( m_released == FALSE &&
-		m_totalFramesPlayed +
-		m_instrumentTrack->eng()->getMixer()->framesPerAudioBuffer() >=
-								m_frames )
+		m_totalFramesPlayed + engine::getMixer()->framesPerPeriod()
+								>= m_frames )
 	{
 		noteOff( m_frames - m_totalFramesPlayed );
 	} 
 
 	// play note!
-	m_instrumentTrack->playNote( this );
+	m_instrumentTrack->playNote( this, _try_parallelizing );
 
 	if( m_released == TRUE )
 	{
-		f_cnt_t todo =
-	m_instrumentTrack->eng()->getMixer()->framesPerAudioBuffer();
+		f_cnt_t todo = engine::getMixer()->framesPerPeriod();
 		// if this note is base-note for arpeggio, always set
 		// m_releaseFramesToDo to bigger value than m_releaseFramesDone
 		// because we do not allow notePlayHandle::done() to be true
@@ -150,7 +185,7 @@ void notePlayHandle::play( void )
 		if( arpBaseNote() == TRUE )
 		{
 			m_releaseFramesToDo = m_releaseFramesDone + 2 *
-		m_instrumentTrack->eng()->getMixer()->framesPerAudioBuffer();
+				engine::getMixer()->framesPerPeriod();
 		}
 		// look whether we have frames left to be done before release
 		if( m_framesBeforeRelease )
@@ -158,7 +193,7 @@ void notePlayHandle::play( void )
 			// yes, then look whether these samples can be played
 			// within one audio-buffer
 			if( m_framesBeforeRelease <=
-		m_instrumentTrack->eng()->getMixer()->framesPerAudioBuffer() )
+				engine::getMixer()->framesPerPeriod() )
 			{
 				// yes, then we did less releaseFramesDone
 				todo -= m_framesBeforeRelease;
@@ -166,12 +201,12 @@ void notePlayHandle::play( void )
 			}
 			else
 			{
-				// no, then just decrese framesBeforeRelease
+				// no, then just decrease framesBeforeRelease
 				// and wait for next loop... (we're not in
 				// release-phase yet)
 				todo = 0;
 				m_framesBeforeRelease -=
-		m_instrumentTrack->eng()->getMixer()->framesPerAudioBuffer();
+				engine::getMixer()->framesPerPeriod();
 			}
 		}
 		// look whether we're in release-phase
@@ -197,7 +232,7 @@ void notePlayHandle::play( void )
 	for( notePlayHandleVector::iterator it = m_subNotes.begin();
 						it != m_subNotes.end(); )
 	{
-		( *it )->play();
+		( *it )->play( _try_parallelizing );
 		if( ( *it )->done() )
 		{
 			delete *it;
@@ -219,34 +254,35 @@ void notePlayHandle::play( void )
 	}
 
 	// update internal data
-	m_totalFramesPlayed +=
-		m_instrumentTrack->eng()->getMixer()->framesPerAudioBuffer();
+	m_totalFramesPlayed += engine::getMixer()->framesPerPeriod();
 }
 
 
 
 
-void notePlayHandle::checkValidity( void )
+f_cnt_t notePlayHandle::framesLeft( void ) const
 {
-	if( m_instrumentTrack != NULL &&
-				m_instrumentTrack->type() == track::NULL_TRACK )
+/*	const instrument * i = ( m_instrumentTrack != NULL ) ?
+				m_instrumentTrack->getInstrument() : NULL;
+	f_cnt_t rftd  = ( i != NULL && i->isMonophonic() ) ?
+						0 : actualReleaseFramesToDo();*/
+	if( m_released && actualReleaseFramesToDo() == 0 )
 	{
-		// track-type being track::NULL_TRACK indicates a track whose
-		// removal is in progress, so we have to invalidate ourself
-		if( m_released == FALSE )
-		{
-			noteOff( 0 );
-		}
-		m_instrumentTrack->deleteNotePluginData( this );
-		m_instrumentTrack = NULL;
+		return( m_framesBeforeRelease );
 	}
-	// sub-notes might not be registered at mixer (for example arpeggio-
-	// notes), so they wouldn't invalidate them-selves
-	for( notePlayHandleVector::iterator it = m_subNotes.begin();
-						it != m_subNotes.end(); ++it )
+	else if( m_released && actualReleaseFramesToDo() >= m_releaseFramesDone )
 	{
-		( *it )->checkValidity();
+		return( m_framesBeforeRelease + actualReleaseFramesToDo() - m_releaseFramesDone );
 	}
+	return( m_frames+actualReleaseFramesToDo()-m_totalFramesPlayed );
+}
+
+
+
+
+bool notePlayHandle::isFromTrack( const track * _track ) const
+{
+	return( m_instrumentTrack == _track || m_bbTrack == _track );
 }
 
 
@@ -263,33 +299,14 @@ void notePlayHandle::noteOff( const f_cnt_t _s )
 
 	// then set some variables indicating release-state
 	m_framesBeforeRelease = _s;
-	if( m_instrumentTrack != NULL )
-	{
-		m_releaseFramesToDo = tMax<f_cnt_t>( 10,
+	m_releaseFramesToDo = tMax<f_cnt_t>( 0, // 10,
 			m_instrumentTrack->m_envWidget->releaseFrames() );
-		if( !configManager::inst()->value( "ui",
-						"manualchannelpiano" ).toInt() )
-		{
-			m_instrumentTrack->m_pianoWidget->setKeyState( key(),
-									FALSE );
-		}
-		// send MIDI-note-off-event
-		// TODO: move framesPerTact() to engine
-		if( m_instrumentTrack->eng()->getSongEditor() )
-		{
-			m_instrumentTrack->processOutEvent( midiEvent( NOTE_OFF,
+	// send MIDI-note-off-event
+	m_instrumentTrack->processOutEvent( midiEvent( NOTE_OFF,
 				m_instrumentTrack->m_midiPort->outputChannel(),
 								key(), 0 ),
-						midiTime::fromFrames(
-							m_framesBeforeRelease,
-			m_instrumentTrack->eng()->getSongEditor()
-							->framesPerTact() ) );
-		}
-	}
-	else
-	{
-		m_releaseFramesToDo = 10;
-	}
+			midiTime::fromFrames( m_framesBeforeRelease,
+						engine::framesPerTact64th() ) );
 
 	m_released = TRUE;
 }
@@ -299,9 +316,8 @@ void notePlayHandle::noteOff( const f_cnt_t _s )
 
 f_cnt_t notePlayHandle::actualReleaseFramesToDo( void ) const
 {
-	return( ( m_instrumentTrack != NULL ) ?
-			m_instrumentTrack->m_envWidget->releaseFrames(
-							arpBaseNote() ) : 0 );
+	return( m_instrumentTrack->m_envWidget->releaseFrames(
+							arpBaseNote() ) );
 }
 
 
@@ -310,10 +326,11 @@ f_cnt_t notePlayHandle::actualReleaseFramesToDo( void ) const
 void notePlayHandle::setFrames( const f_cnt_t _frames )
 {
 	m_frames = _frames;
-	if( m_frames == 0 && m_instrumentTrack != NULL )
+	if( m_frames == 0 )
 	{
 		m_frames = m_instrumentTrack->beatLen( this );
 	}
+	m_orig_frames = m_frames;
 }
 
 
@@ -321,8 +338,7 @@ void notePlayHandle::setFrames( const f_cnt_t _frames )
 
 float notePlayHandle::volumeLevel( const f_cnt_t _frame )
 {
-	return( ( m_instrumentTrack != NULL ) ?
-		m_instrumentTrack->m_envWidget->volumeLevel( this, _frame ) : 0 );
+	return( m_instrumentTrack->m_envWidget->volumeLevel( this, _frame ) );
 }
 
 
@@ -344,8 +360,7 @@ void notePlayHandle::mute( void )
 
 int notePlayHandle::index( void ) const
 {
-	const playHandleVector & phv =
-			m_instrumentTrack->eng()->getMixer()->playHandles();
+	const playHandleVector & phv = engine::getMixer()->playHandles();
 	int idx = 0;
 	for( constPlayHandleVector::const_iterator it = phv.begin();
 							it != phv.end(); ++it )
@@ -371,9 +386,9 @@ int notePlayHandle::index( void ) const
 
 
 constNotePlayHandleVector notePlayHandle::nphsOfInstrumentTrack(
-						const instrumentTrack * _it )
+				const instrumentTrack * _it, bool _all_ph )
 {
-	const playHandleVector & phv = _it->eng()->getMixer()->playHandles();
+	const playHandleVector & phv = engine::getMixer()->playHandles();
 	constNotePlayHandleVector cnphv;
 
 	for( constPlayHandleVector::const_iterator it = phv.begin();
@@ -382,7 +397,7 @@ constNotePlayHandleVector notePlayHandle::nphsOfInstrumentTrack(
 		const notePlayHandle * nph =
 				dynamic_cast<const notePlayHandle *>( *it );
 		if( nph != NULL && nph->m_instrumentTrack == _it &&
-						nph->released() == FALSE )
+			( nph->released() == FALSE || _all_ph == TRUE ) )
 		{
 			cnphv.push_back( nph );
 		}
@@ -392,6 +407,7 @@ constNotePlayHandleVector notePlayHandle::nphsOfInstrumentTrack(
 
 
 
+
 bool notePlayHandle::operator==( const notePlayHandle & _nph ) const
 {
 	return( length() == _nph.length() &&
@@ -401,7 +417,7 @@ bool notePlayHandle::operator==( const notePlayHandle & _nph ) const
 			getPanning() == _nph.getPanning() &&
 			m_instrumentTrack == _nph.m_instrumentTrack &&
 			m_frames == _nph.m_frames &&
-			m_framesAhead == _nph.m_framesAhead &&
+			offset() == _nph.offset() &&
 			m_totalFramesPlayed == _nph.m_totalFramesPlayed &&
 			m_released == _nph.m_released &&
 			m_baseNote == _nph.m_baseNote &&
@@ -414,7 +430,17 @@ bool notePlayHandle::operator==( const notePlayHandle & _nph ) const
 
 void notePlayHandle::updateFrequency( void )
 {
-	m_frequency = m_instrumentTrack->frequency( this );
+	float pitch = (float)( tone() - m_instrumentTrack->baseTone() +
+			engine::getSongEditor()->masterPitch() ) / 12.0f +
+			(float)( octave() - m_instrumentTrack->baseOctave() )
+			+ m_base_detuning->value() / 12.0f;
+	m_frequency = BASE_FREQ * powf( 2.0f, pitch );
+
+	for( notePlayHandleVector::iterator it = m_subNotes.begin();
+						it != m_subNotes.end(); ++it )
+	{
+		( *it )->updateFrequency();
+	}
 }
 
 
@@ -422,13 +448,36 @@ void notePlayHandle::updateFrequency( void )
 
 void notePlayHandle::processMidiTime( const midiTime & _time )
 {
-	detuning()->getAutomationPattern()->processMidiTime( _time - pos() );
+	if( _time >= pos() )
+	{
+		int level = detuning()->getAutomationPattern()->valueAt( _time -
+									pos() );
+		if( level != m_base_detuning->level() )
+		{
+			m_base_detuning->setLevel( level );
+			updateFrequency();
+		}
+	}
 }
 
 
 
 
-#include "note_play_handle.moc"
+void notePlayHandle::resize( const bpm_t _new_bpm )
+{
+	double completed = m_totalFramesPlayed / (double)m_frames;
+	double new_frames = m_orig_frames * m_orig_bpm / (double)_new_bpm;
+	m_frames = (f_cnt_t)new_frames;
+	m_totalFramesPlayed = (f_cnt_t)( completed * new_frames );
+
+	for( notePlayHandleVector::iterator it = m_subNotes.begin();
+						it != m_subNotes.end(); ++it )
+	{
+		( *it )->resize( _new_bpm );
+	}
+}
+
+
 
 
 #endif
