@@ -4,7 +4,7 @@
  * piano_roll.cpp - implementation of piano-roll which is used for actual
  *                  writing of melodies
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -33,6 +33,7 @@
 #include <Qt/QtXml>
 #include <QtGui/QApplication>
 #include <QtGui/QButtonGroup>
+#include <QtGui/QClipboard>
 #include <QtGui/QPainter>
 #include <QtGui/QKeyEvent>
 #include <QtGui/QWheelEvent>
@@ -43,10 +44,12 @@
 
 #include <qapplication.h>
 #include <qbuttongroup.h>
-#include <qpainter.h>
-#include <qlayout.h>
-#include <qlabel.h>
+#include <qclipboard.h>
 #include <qdom.h>
+#include <qdragobject.h>
+#include <qlabel.h>
+#include <qlayout.h>
+#include <qpainter.h>
 
 #define addButton insert
 #define setCheckable setToggleButton
@@ -62,21 +65,29 @@
 
 
 #include "piano_roll.h"
-#include "song_editor.h"
-#include "main_window.h"
-#include "pattern.h"
-#include "embed.h"
-#include "pixmap_button.h"
-#include "templates.h"
-#include "gui_templates.h"
-#include "timeline.h"
-#include "instrument_track.h"
-#include "tooltip.h"
-#include "midi.h"
-#include "tool_button.h"
-#include "text_float.h"
+#include "automatable_object_templates.h"
+#include "clipboard.h"
 #include "combobox.h"
+#include "debug.h"
+#include "detuning_helper.h"
+#include "embed.h"
+#include "gui_templates.h"
+#include "instrument_track.h"
+#include "main_window.h"
+#include "midi.h"
+#include "mmp.h"
+#include "pattern.h"
 #include "piano_widget.h"
+#include "pixmap_button.h"
+#include "song_editor.h"
+#include "templates.h"
+#include "text_float.h"
+#include "timeline.h"
+#include "tool_button.h"
+#include "tooltip.h"
+
+
+typedef automationPattern::timeMap timeMap;
 
 
 extern tones whiteKeys[];	// defined in piano_widget.cpp
@@ -102,7 +113,7 @@ const int PR_BOTTOM_MARGIN = SCROLLBAR_SIZE;
 const int PR_TOP_MARGIN = 48;
 
 // width of area used for resizing (the grip at the end of a note)
-const int RESIZE_AREA_WIDTH = 3;
+const int RESIZE_AREA_WIDTH = 4;
 
 // width of line for setting volume/panning of note
 const int NE_LINE_WIDTH = 3;
@@ -132,11 +143,8 @@ pianoRoll::pianoRollKeyTypes pianoRoll::prKeyOrder[] =
 const int DEFAULT_PR_PPT = KEY_LINE_HEIGHT * DEFAULT_STEPS_PER_TACT;
 
 
-pianoRoll::pianoRoll( engine * _engine ) :
-	QWidget( _engine->getMainWindow()->workspace() ),
-	journallingObject( _engine ),
-	m_paintPixmap(),
-	m_cursorInside( FALSE ),
+pianoRoll::pianoRoll( void ) :
+	QWidget( engine::getMainWindow()->workspace() ),
 	m_pattern( NULL ),
 	m_currentPosition(),
 	m_recording( FALSE ),
@@ -148,7 +156,6 @@ pianoRoll::pianoRoll( engine * _engine ) :
 	m_ppt( DEFAULT_PR_PPT ),
 	m_lenOfNewNotes( midiTime( 0, 16 ) ),
 	m_startKey( INITIAL_START_KEY ),
-	m_keyMouseOver( INITIAL_START_KEY ),
 	m_lastKey( 0 ),
 	m_editMode( DRAW ),
 	m_scrollBack( FALSE )
@@ -197,15 +204,14 @@ pianoRoll::pianoRoll( engine * _engine ) :
 
 #ifdef QT4
 	// add us to workspace
-	eng()->getMainWindow()->workspace()->addWindow( this );
+	engine::getMainWindow()->workspace()->addWindow( this );
 #endif
 
 	// add time-line
 	m_timeLine = new timeLine( WHITE_KEY_WIDTH, 32, m_ppt,
-					eng()->getSongEditor()->getPlayPos(
+					engine::getSongEditor()->getPlayPos(
 						songEditor::PLAY_PATTERN ),
-						m_currentPosition, this,
-								eng() );
+						m_currentPosition, this );
 	connect( this, SIGNAL( positionChanged( const midiTime & ) ),
 		m_timeLine, SLOT( updatePosition( const midiTime & ) ) );
 	connect( m_timeLine, SIGNAL( positionChanged( const midiTime & ) ),
@@ -404,7 +410,7 @@ pianoRoll::pianoRoll( engine * _engine ) :
 	zoom_lbl->setPixmap( embed::getIconPixmap( "zoom" ) );
 
 	// setup zooming-stuff
-	m_zoomingComboBox = new comboBox( m_toolBar, NULL, eng(), NULL );
+	m_zoomingComboBox = new comboBox( m_toolBar, NULL, NULL );
 	m_zoomingComboBox->setFixedSize( 80, 22 );
 	for( int i = 0; i < 6; ++i )
 	{
@@ -420,7 +426,7 @@ pianoRoll::pianoRoll( engine * _engine ) :
 	QLabel * quantize_lbl = new QLabel( m_toolBar );
 	quantize_lbl->setPixmap( embed::getIconPixmap( "quantize" ) );
 
-	m_quantizeComboBox = new comboBox( m_toolBar, NULL, eng(), NULL );
+	m_quantizeComboBox = new comboBox( m_toolBar, NULL, NULL );
 	m_quantizeComboBox->setFixedSize( 60, 22 );
 	for( int i = 0; i < 7; ++i )
 	{
@@ -433,7 +439,7 @@ pianoRoll::pianoRoll( engine * _engine ) :
 	QLabel * note_len_lbl = new QLabel( m_toolBar );
 	note_len_lbl->setPixmap( embed::getIconPixmap( "note" ) );
 
-	m_noteLenComboBox = new comboBox( m_toolBar, NULL, eng(), NULL );
+	m_noteLenComboBox = new comboBox( m_toolBar, NULL, NULL );
 	m_noteLenComboBox->setFixedSize( 120, 22 );
 	m_noteLenComboBox->addItem( tr( "Last note" ),
 					embed::getIconPixmap( "edit_draw" ) );
@@ -523,13 +529,13 @@ void pianoRoll::setCurrentPattern( pattern * _new_pattern )
 	}
 
 
-	noteVector & notes = m_pattern->notes();
+	const noteVector & notes = m_pattern->notes();
 	int central_key = 0;
 	if( notes.empty() == FALSE )
 	{
 		// determine the central key so that we can scroll to it
 		int total_notes = 0;
-		for( noteVector::iterator it = notes.begin();
+		for( noteVector::const_iterator it = notes.begin();
 						it != notes.end(); ++it )
 		{
 			if( ( *it )->length() > 0 )
@@ -633,14 +639,43 @@ inline void pianoRoll::drawNoteRect( QPainter & _p, Uint16 _x, Uint16 _y,
 
 
 
-void pianoRoll::update( void )
+inline void pianoRoll::drawDetuningInfo( QPainter & _p, note * _n, Uint16 _x,
+								Uint16 _y )
 {
-	if( m_paintPixmap.isNull() == TRUE || m_paintPixmap.size() != size() )
+	Uint16 middle_y = _y + KEY_LINE_HEIGHT / 2;
+	_p.setPen( QColor( 0xFF, 0xDF, 0x20 ) );
+
+	timeMap & map = _n->detuning()->getAutomationPattern()->getTimeMap();
+	timeMap::iterator it = map.end();
+	do
 	{
-		m_paintPixmap = QPixmap( size() );
-	}
-	m_paintPixmap.fill( QColor( 0, 0, 0 ) );
-	QPainter p( &m_paintPixmap );
+		--it;
+		Sint32 pos_tact_64th = -it.key();
+		if( pos_tact_64th > _n->length() )
+		{
+			break;
+		}
+		Uint16 pos_x = _x + pos_tact_64th * m_ppt / 64;
+
+#ifdef QT3
+		const int level = it.data();
+#else
+		const int level = it.value();
+#endif
+		Uint16 pos_y = middle_y - level * KEY_LINE_HEIGHT / 10;
+
+		_p.drawLine( pos_x - 1, pos_y, pos_x + 1, pos_y );
+		_p.drawLine( pos_x, pos_y - 1, pos_x, pos_y + 1 );
+	} while( it != map.begin() );
+}
+
+
+
+
+void pianoRoll::updatePaintPixmap( QPixmap & _p )
+{
+	_p.fill( QColor( 0, 0, 0 ) );
+	QPainter p( &_p );
 
 	// set font-size to 8
 	p.setFont( pointSize<8>( p.font() ) );
@@ -870,14 +905,20 @@ void pianoRoll::update( void )
 	int y_base = height() - PR_BOTTOM_MARGIN - m_notesEditHeight - 1;
 	if( validPattern() == TRUE )
 	{
-		noteVector & notes = m_pattern->notes();
+		QPainter p_detuning( &_p );
+		p_detuning.setClipRect( WHITE_KEY_WIDTH, PR_TOP_MARGIN,
+				width() - WHITE_KEY_WIDTH,
+				height() - PR_TOP_MARGIN - PR_BOTTOM_MARGIN
+							- m_notesEditHeight );
+
+		const noteVector & notes = m_pattern->notes();
 
 		const int visible_keys = ( height() - PR_TOP_MARGIN -
 					PR_BOTTOM_MARGIN - m_notesEditHeight ) /
 							KEY_LINE_HEIGHT + 2;
 
-		for( noteVector::iterator it = notes.begin(); it != notes.end();
-									++it )
+		for( noteVector::const_iterator it = notes.begin();
+						it != notes.end(); ++it )
 		{
 			Sint32 len_tact_64th = ( *it )->length();
 
@@ -946,6 +987,13 @@ void pianoRoll::update( void )
 						( *it )->getVolume() / 2,
 					x + WHITE_KEY_WIDTH + 1,
 					height() - PR_BOTTOM_MARGIN );
+
+			if( ( *it )->hasDetuningInfo() )
+			{
+				drawDetuningInfo( p_detuning, *it,
+					x + WHITE_KEY_WIDTH,
+					y_base - key * KEY_LINE_HEIGHT );
+			}
 		}
 	}
 	else
@@ -982,8 +1030,6 @@ void pianoRoll::update( void )
 #else
 	m_leftRightScroll->setSteps( 1, l );
 #endif
-
-	QWidget::update();
 }
 
 
@@ -1010,15 +1056,6 @@ void pianoRoll::closeEvent( QCloseEvent * _ce )
 
 
 
-void pianoRoll::enterEvent( QEvent * _e )
-{
-	m_cursorInside = TRUE;
-	QWidget::enterEvent( _e );
-}
-
-
-
-
 void pianoRoll::keyPressEvent( QKeyEvent * _ke )
 {
 	if( validPattern() )
@@ -1150,7 +1187,7 @@ void pianoRoll::keyPressEvent( QKeyEvent * _ke )
 			break;
 
 		case Qt::Key_Space:
-			if( eng()->getSongEditor()->playing() )
+			if( engine::getSongEditor()->playing() )
 			{
 				stop();
 			}
@@ -1166,13 +1203,18 @@ void pianoRoll::keyPressEvent( QKeyEvent * _ke )
 			m_timeLine->updatePosition();
 			break;
 
-		case Key_Control:
+		case Qt::Key_Control:
 			if( mouseOverNote() )
 			{
 				m_editMode = OPEN;
+#ifndef QT3
+				QApplication::changeOverrideCursor(
+						QCursor( Qt::ArrowCursor ) );
+#else
 				QApplication::setOverrideCursor(
-						QCursor( ArrowCursor ), TRUE );
-				QWidget::update();
+					QCursor( Qt::ArrowCursor ), TRUE );
+#endif
+				update();
 			}
 
 		default:
@@ -1193,7 +1235,7 @@ void pianoRoll::keyReleaseEvent( QKeyEvent * _ke )
 	}
 	switch( _ke->key() )
 	{
-		case Key_Control:
+		case Qt::Key_Control:
 			if( m_editMode == OPEN )
 			{
 				m_editMode = DRAW;
@@ -1212,7 +1254,6 @@ void pianoRoll::leaveEvent( QEvent * _e )
 	{
 		QApplication::restoreOverrideCursor();
 	}
-	m_cursorInside = FALSE;
 
 	QWidget::leaveEvent( _e );
 }
@@ -1256,10 +1297,10 @@ void pianoRoll::mousePressEvent( QMouseEvent * _me )
 							m_currentPosition;
 
 			// get note-vector of current pattern
-			noteVector & notes = m_pattern->notes();
+			const noteVector & notes = m_pattern->notes();
 
 			// will be our iterator in the following loop
-			noteVector::iterator it = notes.begin();
+			noteVector::const_iterator it = notes.begin();
 
 			// loop through whole note-vector...
 			while( it != notes.end() )
@@ -1323,8 +1364,7 @@ void pianoRoll::mousePressEvent( QMouseEvent * _me )
 					midiTime note_pos( pos_tact_64th );
 					midiTime note_len( newNoteLen() );
 		
-					note new_note( eng(),
-							note_len, note_pos,
+					note new_note( note_len, note_pos,
 							(tones)( key_num %
 							NOTES_PER_OCTAVE ),
 							(octaves)( key_num /
@@ -1347,8 +1387,9 @@ void pianoRoll::mousePressEvent( QMouseEvent * _me )
 				m_currentNote = *it;
 
 				// clicked at the "tail" of the note?
-				if( pos_tact_64th > m_currentNote->pos() +
-						m_currentNote->length() -
+				if( pos_tact_64th*m_ppt/64 >
+					( m_currentNote->pos() +
+					m_currentNote->length() )*m_ppt/64 -
 							RESIZE_AREA_WIDTH &&
 						m_currentNote->length() > 0 )
 				{
@@ -1374,7 +1415,7 @@ void pianoRoll::mousePressEvent( QMouseEvent * _me )
 					QApplication::setOverrideCursor( c );
 				}
 
-				eng()->getSongEditor()->setModified();
+				engine::getSongEditor()->setModified();
 			}
 			else if( ( _me->button() == Qt::RightButton &&
 							m_editMode == DRAW ) ||
@@ -1394,7 +1435,7 @@ void pianoRoll::mousePressEvent( QMouseEvent * _me )
 						( *it )->setLength( 0 );
 						m_pattern->update();
 					}
-					eng()->getSongEditor()->setModified();
+					engine::getSongEditor()->setModified();
 				}
 			}
 			else if( _me->button() == Qt::LeftButton &&
@@ -1433,7 +1474,7 @@ void pianoRoll::mousePressEvent( QMouseEvent * _me )
 				m_action = MOVE_SELECTION;
 
 				play_note = FALSE;
-				eng()->getSongEditor()->setModified();
+				engine::getSongEditor()->setModified();
 			}
 			else if( _me->button() == Qt::RightButton &&
 							m_editMode == MOVE )
@@ -1449,8 +1490,9 @@ void pianoRoll::mousePressEvent( QMouseEvent * _me )
 
 		// was there an action where should be played the note?
 		if( play_note == TRUE && m_recording == FALSE &&
-				eng()->getSongEditor()->playing() == FALSE )
+				engine::getSongEditor()->playing() == FALSE )
 		{
+			m_lastKey = key_num;
 			m_pattern->getInstrumentTrack()->processInEvent(
 					midiEvent( NOTE_ON, 0, key_num,
 							vol * 127 / 100 ),
@@ -1475,7 +1517,7 @@ void pianoRoll::mouseReleaseEvent( QMouseEvent * _me )
 		else
 		{
 			m_pattern->getInstrumentTrack()->processInEvent(
-				midiEvent( NOTE_OFF, 0, getKey( _me->y() ), 0 ),
+					midiEvent( NOTE_OFF, 0, m_lastKey, 0 ),
 								midiTime() );
 		}
 	}
@@ -1501,22 +1543,18 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 		return;
 	}
 
-	// save current last-key-var
-	int released_key = m_lastKey;
-
 	if( _me->y() > PR_TOP_MARGIN )
 	{
 		bool edit_note = ( _me->y() > height() -
 					PR_BOTTOM_MARGIN - m_notesEditHeight );
 
 		int key_num = getKey( _me->y() );
-		m_keyMouseOver = key_num;
 		int x = _me->x();
 
 		// is the calculated key different from current key?
 		// (could be the user just moved the cursor one pixel up/down
 		// but is still on the same key)
-		if( key_num != released_key &&
+		if( key_num != m_lastKey &&
 			m_action != CHANGE_NOTE_VOLUME &&
 			m_action != MOVE_SELECTION &&
 			edit_note == FALSE &&
@@ -1528,7 +1566,7 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 					Qt::LeftButton )
 		{
 			m_pattern->getInstrumentTrack()->processInEvent(
-				midiEvent( NOTE_OFF, 0, released_key, 0 ),
+					midiEvent( NOTE_OFF, 0, m_lastKey, 0 ),
 								midiTime() );
 			if(
 #ifdef QT4
@@ -1541,8 +1579,9 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 				m_action != SELECT_NOTES &&
 				m_action != MOVE_SELECTION &&
 				m_recording == FALSE &&
-				eng()->getSongEditor()->playing() == FALSE )
+				engine::getSongEditor()->playing() == FALSE )
 			{
+				m_lastKey = key_num;
 				m_pattern->getInstrumentTrack()->processInEvent(
 					midiEvent( NOTE_ON, 0, key_num,
 						DEFAULT_VOLUME * 127 / 100 ),
@@ -1551,7 +1590,7 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 		}
 		if( _me->x() <= WHITE_KEY_WIDTH )
 		{
-			QWidget::update();
+			update();
 			return;
 		}
 		x -= WHITE_KEY_WIDTH;
@@ -1623,12 +1662,12 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 				m_pattern->update();
 			}
 
-			eng()->getSongEditor()->setModified();
+			engine::getSongEditor()->setModified();
 
 		}
 		else if(
 #ifdef QT4
-			_me->buttons() == NoButton
+			_me->buttons() == Qt::NoButton
 #else
 			( _me->state() == NoButton
 					|| _me->state() == ControlButton )
@@ -1642,10 +1681,10 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 							m_currentPosition;
 
 			// get note-vector of current pattern
-			noteVector & notes = m_pattern->notes();
+			const noteVector & notes = m_pattern->notes();
 
 			// will be our iterator in the following loop
-			noteVector::iterator it = notes.begin();
+			noteVector::const_iterator it = notes.begin();
 
 			// loop through whole note-vector...
 			while( it != notes.end() )
@@ -1667,16 +1706,23 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 			// no note??
 			if( it != notes.end() )
 			{
-				if( _me->modifiers() & ControlModifier )
+				if( _me->modifiers() & Qt::ControlModifier )
 				{
 					m_editMode = OPEN;
+#ifndef QT3
+					QApplication::changeOverrideCursor(
+						QCursor( Qt::ArrowCursor ) );
+#else
 					QApplication::setOverrideCursor(
-						QCursor( ArrowCursor ), TRUE );
+						QCursor( Qt::ArrowCursor ),
+									TRUE );
+#endif
 				}
 				// cursor at the "tail" of the note?
 				else if( ( *it )->length() > 0 &&
-					pos_tact_64th > ( *it )->pos() +
-						( *it )->length() -
+					pos_tact_64th*m_ppt/64 >
+						( ( *it )->pos() +
+						( *it )->length() )*m_ppt/64 -
 							RESIZE_AREA_WIDTH )
 				{
 					if( QApplication::overrideCursor() )
@@ -1875,15 +1921,11 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 				int note_tact_64th =
 						( *it )->pos().getTact64th() +
 								tact_64th_diff;
-				while( note_tact_64th < 0 )
+				// ensure note_tact_64th range
+				if( note_tact_64th >> 6 )
 				{
-					--note_tact;
-					note_tact_64th += 64;
-				}
-				while( note_tact_64th > 64 )
-				{
-					++note_tact;
-					note_tact_64th -= 64;
+					note_tact += note_tact_64th >> 6;
+					note_tact_64th &= 63;
 				}
 				midiTime new_note_pos( note_tact,
 							note_tact_64th );
@@ -1896,7 +1938,7 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 			m_moveStartKey = key_num;
 		}
 		else if( m_editMode == OPEN && !( mouseOverNote()
-				&& _me->modifiers() & ControlModifier ) )
+				&& _me->modifiers() & Qt::ControlModifier ) )
 		{
 			m_editMode = DRAW;
 		}
@@ -1989,20 +2031,7 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 		QApplication::restoreOverrideCursor();
 	}
 
-	if(
-#ifdef QT4
-			_me->buttons() &
-#else
-			_me->state() ==
-#endif
-						Qt::NoButton )
-	{
-		QWidget::update();
-	}
-	else
-	{
-		update();
-	}
+	update();
 }
 
 
@@ -2010,6 +2039,8 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 
 void pianoRoll::paintEvent( QPaintEvent * )
 {
+	QPixmap paintPixmap( size() );
+	updatePaintPixmap( paintPixmap );
 #ifdef QT4
 	QPainter p( this );
 #else
@@ -2018,36 +2049,34 @@ void pianoRoll::paintEvent( QPaintEvent * )
 
 	QPainter p( &draw_pm, this );
 #endif
-	p.drawPixmap( 0, 0, m_paintPixmap );
+	p.drawPixmap( 0, 0, paintPixmap );
 
-	if( m_cursorInside == TRUE )
-	{
-		p.setClipRect( WHITE_KEY_WIDTH, PR_TOP_MARGIN, width() -
+	p.setClipRect( WHITE_KEY_WIDTH, PR_TOP_MARGIN, width() -
 				WHITE_KEY_WIDTH, height() - PR_TOP_MARGIN -
 					m_notesEditHeight - PR_BOTTOM_MARGIN );
-		if( validPattern() == TRUE )
-		{
-			p.fillRect( 10, height() + 3 - PR_BOTTOM_MARGIN -
+	if( validPattern() == TRUE )
+	{
+		int key_num = getKey( mapFromGlobal( QCursor::pos() ).y() );
+		p.fillRect( 10, height() + 3 - PR_BOTTOM_MARGIN -
 				m_notesEditHeight - KEY_LINE_HEIGHT *
-					( m_keyMouseOver - m_startKey + 1 ),
+					( key_num - m_startKey + 1 ),
 				width() - 10, KEY_LINE_HEIGHT - 7,
 							QColor( 64, 64, 64 ) );
-		}
-
-		const QPixmap * cursor = NULL;
-		// draw current edit-mode-icon below the cursor
-		switch( m_editMode )
-		{
-			case DRAW: cursor = s_toolDraw; break;
-			case ERASE: cursor = s_toolErase; break;
-			case SELECT: cursor = s_toolSelect; break;
-			case MOVE: cursor = s_toolMove; break;
-			case OPEN: cursor = s_toolOpen; break;
-		}
-		p.drawPixmap( mapFromGlobal( QCursor::pos() ) + QPoint( 8, 8 ),
-								*cursor );
 	}
 
+	const QPixmap * cursor = NULL;
+	// draw current edit-mode-icon below the cursor
+	switch( m_editMode )
+	{
+		case DRAW: cursor = s_toolDraw; break;
+		case ERASE: cursor = s_toolErase; break;
+		case SELECT: cursor = s_toolSelect; break;
+		case MOVE: cursor = s_toolMove; break;
+		case OPEN: cursor = s_toolOpen; break;
+	}
+	p.drawPixmap( mapFromGlobal( QCursor::pos() ) + QPoint( 8, 8 ),
+								*cursor );
+
 #ifndef QT4
 	// and blit all the drawn stuff on the screen...
 	bitBlt( this, rect().topLeft(), &draw_pm );
@@ -2088,7 +2117,7 @@ void pianoRoll::resizeEvent( QResizeEvent * )
 	}
 	m_topBottomScroll->setValue( m_totalKeysToScroll - m_startKey );
 
-	eng()->getSongEditor()->getPlayPos( songEditor::PLAY_PATTERN
+	engine::getSongEditor()->getPlayPos( songEditor::PLAY_PATTERN
 					).m_timeLine->setFixedWidth( width() );
 	m_toolBar->setFixedWidth( width() );
 	update();
@@ -2100,7 +2129,7 @@ void pianoRoll::resizeEvent( QResizeEvent * )
 void pianoRoll::wheelEvent( QWheelEvent * _we )
 {
 	_we->accept();
-	if( eng()->getMainWindow()->isCtrlPressed() == TRUE )
+	if( engine::getMainWindow()->isCtrlPressed() == TRUE )
 	{
 		if( _we->delta() > 0 )
 		{
@@ -2120,7 +2149,7 @@ void pianoRoll::wheelEvent( QWheelEvent * _we )
 		m_timeLine->setPixelsPerTact( m_ppt );
 		update();
 	}
-	else if( eng()->getMainWindow()->isShiftPressed() )
+	else if( engine::getMainWindow()->isShiftPressed() )
 	{
 		m_leftRightScroll->setValue( m_leftRightScroll->value() -
 							_we->delta() * 2 / 15 );
@@ -2135,7 +2164,7 @@ void pianoRoll::wheelEvent( QWheelEvent * _we )
 
 
 
-int pianoRoll::getKey( int _y )
+int pianoRoll::getKey( int _y ) const
 {
 	int key_line_y = height() - PR_BOTTOM_MARGIN - m_notesEditHeight - 1;
 	// pressed key on piano
@@ -2153,7 +2182,7 @@ int pianoRoll::getKey( int _y )
 		key_num = NOTES_PER_OCTAVE * OCTAVES - 1;
 	}
 
-	return( m_lastKey = key_num );
+	return( key_num );
 }
 
 
@@ -2166,31 +2195,31 @@ void pianoRoll::play( void )
 		return;
 	}
 
-	if( eng()->getSongEditor()->playing() )
+	if( engine::getSongEditor()->playing() )
 	{
-		if( eng()->getSongEditor()->playMode() !=
+		if( engine::getSongEditor()->playMode() !=
 						songEditor::PLAY_PATTERN )
 		{
-			eng()->getSongEditor()->stop();
-			eng()->getSongEditor()->playPattern( m_pattern );
+			engine::getSongEditor()->stop();
+			engine::getSongEditor()->playPattern( m_pattern );
 			m_playButton->setIcon( embed::getIconPixmap(
 								"pause" ) );
 		}
 		else
 		{
-			eng()->getSongEditor()->pause();
+			engine::getSongEditor()->pause();
 			m_playButton->setIcon( embed::getIconPixmap( "play" ) );
 		}
 	}
-	else if( eng()->getSongEditor()->paused() )
+	else if( engine::getSongEditor()->paused() )
 	{
-		eng()->getSongEditor()->resumeFromPause();
+		engine::getSongEditor()->resumeFromPause();
 		m_playButton->setIcon( embed::getIconPixmap( "pause" ) );
 	}
 	else
 	{
 		m_playButton->setIcon( embed::getIconPixmap( "pause" ) );
-		eng()->getSongEditor()->playPattern( m_pattern );
+		engine::getSongEditor()->playPattern( m_pattern );
 	}
 }
 
@@ -2199,7 +2228,7 @@ void pianoRoll::play( void )
 
 void pianoRoll::record( void )
 {
-	if( eng()->getSongEditor()->playing() )
+	if( engine::getSongEditor()->playing() )
 	{
 		stop();
 	}
@@ -2209,7 +2238,7 @@ void pianoRoll::record( void )
 	}
 
 	m_recording = TRUE;
-	eng()->getSongEditor()->playPattern( m_pattern, FALSE );
+	engine::getSongEditor()->playPattern( m_pattern, FALSE );
 }
 
 
@@ -2217,7 +2246,7 @@ void pianoRoll::record( void )
 
 void pianoRoll::stop( void )
 {
-	eng()->getSongEditor()->stop();
+	engine::getSongEditor()->stop();
 	m_playButton->setIcon( embed::getIconPixmap( "play" ) );
 	m_playButton->update();
 	m_recording = FALSE;
@@ -2231,19 +2260,14 @@ void pianoRoll::recordNote( const note & _n )
 {
 	if( m_recording == TRUE && validPattern() == TRUE )
 	{
-		note n( _n );
-		n.setPos( eng()->getSongEditor()->getPlayPos(
-				songEditor::PLAY_PATTERN ) - n.length() );
+		note n( _n.length(), engine::getSongEditor()->getPlayPos(
+				songEditor::PLAY_PATTERN ) - _n.length(),
+				_n.tone(), _n.octave(),
+				_n.getVolume(), _n.getPanning() );
 		n.quantizeLength( quantization() );
-#ifndef QT4
-		qApp->lock();
-#endif
 		m_pattern->addNote( n );
 		update();
-#ifndef QT4
-		qApp->unlock();
-#endif
-		eng()->getSongEditor()->setModified();
+		engine::getSongEditor()->setModified();
 	}
 }
 
@@ -2319,12 +2343,13 @@ void pianoRoll::selectAll( void )
 		return;
 	}
 
-	noteVector & notes = m_pattern->notes();
+	const noteVector & notes = m_pattern->notes();
 
 	// if first_time = TRUE, we HAVE to set the vars for select
 	bool first_time = TRUE;
 
-	for( noteVector::iterator it = notes.begin(); it != notes.end(); ++it )
+	for( noteVector::const_iterator it = notes.begin(); it != notes.end();
+									++it )
 	{
 		Uint32 len_tact_64th = ( *it )->length();
 
@@ -2339,9 +2364,9 @@ void pianoRoll::selectAll( void )
 				// the difference between old and new start-key
 				// to m_selectedKeys, otherwise the selection
 				// is just moved down...
-				int diff = m_selectStartKey - ( key - 1 );
+				m_selectedKeys += m_selectStartKey
+								- ( key - 1 );
 				m_selectStartKey = key - 1;
-				m_selectedKeys += diff;
 			}
 			if( key >= m_selectedKeys+m_selectStartKey ||
 								first_time )
@@ -2391,9 +2416,10 @@ void pianoRoll::getSelectedNotes( noteVector & _selected_notes )
 		qSwap<int>( sel_key_start, sel_key_end );
 	}
 
-	noteVector & notes = m_pattern->notes();
+	const noteVector & notes = m_pattern->notes();
 
-	for( noteVector::iterator it = notes.begin(); it != notes.end(); ++it )
+	for( noteVector::const_iterator it = notes.begin(); it != notes.end();
+									++it )
 	{
 		Sint32 len_tact_64th = ( *it )->length();
 
@@ -2416,31 +2442,46 @@ void pianoRoll::getSelectedNotes( noteVector & _selected_notes )
 
 
 
-void pianoRoll::copySelectedNotes( void )
+void pianoRoll::copy_to_clipboard( const noteVector & _notes ) const
 {
-	for( noteVector::iterator it = m_notesToCopy.begin();
-					it != m_notesToCopy.end(); ++it )
+	multimediaProject mmp( multimediaProject::CLIPBOARD_DATA );
+	QDomElement note_list = mmp.createElement( "note-list" );
+	mmp.content().appendChild( note_list );
+
+	midiTime start_pos( _notes.front()->pos().getTact(), 0 );
+	for( noteVector::const_iterator it = _notes.begin(); it != _notes.end();
+									++it )
 	{
-		delete *it;
+		note clip_note( **it );
+		clip_note.setPos( clip_note.pos( start_pos ) );
+		clip_note.saveState( mmp, note_list );
 	}
 
-	m_notesToCopy.clear();
+#ifdef QT4
+	QMimeData * clip_content = new QMimeData;
+	clip_content->setData( clipboard::mimeType(), mmp.toString().toUtf8() );
+	QApplication::clipboard()->setMimeData( clip_content,
+							QClipboard::Clipboard );
+#else
+	QStoredDrag * clip_content = new QStoredDrag( clipboard::mimeType() );
+	clip_content->setEncodedData( mmp.toString().utf8() );
+	QApplication::clipboard()->setData( clip_content,
+							QClipboard::Clipboard );
+#endif
+}
 
+
+
+
+void pianoRoll::copySelectedNotes( void )
+{
 	noteVector selected_notes;
 	getSelectedNotes( selected_notes );
 
 	if( selected_notes.empty() == FALSE )
 	{
-		midiTime start_pos( selected_notes.front()->pos().getTact(),
-									0 );
-		for( noteVector::iterator it = selected_notes.begin();
-			it != selected_notes.end(); ++it )
-		{
-			m_notesToCopy.push_back( new note( **it ) );
-			m_notesToCopy.back()->setPos( m_notesToCopy.back()->pos(
-								start_pos ) );
-			m_notesToCopy.back()->detachCurrentDetuning();
-		}
+		copy_to_clipboard( selected_notes );
+
 		textFloat::displayMessage( tr( "Notes copied" ),
 				tr( "All selected notes were copied to the "
 								"clipboard." ),
@@ -2458,39 +2499,26 @@ void pianoRoll::cutSelectedNotes( void )
 		return;
 	}
 
-	for( noteVector::iterator it = m_notesToCopy.begin();
-					it != m_notesToCopy.end(); ++it )
-	{
-		delete *it;
-	}
-
-	m_notesToCopy.clear();
-
 	noteVector selected_notes;
 	getSelectedNotes( selected_notes );
 
 	if( selected_notes.empty() == FALSE )
 	{
-		eng()->getSongEditor()->setModified();
+		copy_to_clipboard( selected_notes );
 
-		midiTime start_pos( selected_notes.front()->pos().getTact(),
-									0 );
+		engine::getSongEditor()->setModified();
 
-		while( selected_notes.empty() == FALSE )
+		for( noteVector::iterator it = selected_notes.begin();
+					it != selected_notes.end(); ++it )
 		{
-			note * new_note = new note( *selected_notes.front() );
-			new_note->setPos( new_note->pos( start_pos ) );
-			m_notesToCopy.push_back( new_note );
-
 			// note (the memory of it) is also deleted by
 			// pattern::removeNote(...) so we don't have to do that
-			m_pattern->removeNote( selected_notes.front() );
-			selected_notes.erase( selected_notes.begin() );
+			m_pattern->removeNote( *it );
 		}
 	}
 
 	update();
-	eng()->getSongEditor()->update();
+	engine::getSongEditor()->update();
 }
 
 
@@ -2503,22 +2531,34 @@ void pianoRoll::pasteNotes( void )
 		return;
 	}
 
-	if( m_notesToCopy.empty() == FALSE )
+#ifdef QT4
+	QString value = QApplication::clipboard()
+				->mimeData( QClipboard::Clipboard )
+						->data( clipboard::mimeType() );
+#else
+	QString value = QApplication::clipboard()->data( QClipboard::Clipboard )
+					->encodedData( clipboard::mimeType() );
+#endif
+
+	if( !value.isEmpty() )
 	{
-		for( noteVector::iterator it = m_notesToCopy.begin();
-					it != m_notesToCopy.end(); ++it )
+		multimediaProject mmp( value, FALSE );
+
+		QDomNodeList list = mmp.elementsByTagName(
+							note::classNodeName() );
+		for( int i = 0; !list.item( i ).isNull(); ++i )
 		{
-			note cur_note( **it );
+			note cur_note;
+			cur_note.restoreState( list.item( i ).toElement() );
 			cur_note.setPos( cur_note.pos() + m_currentPosition );
-			cur_note.detachCurrentDetuning();
 			m_pattern->addNote( cur_note );
 		}
 
 		// we only have to do the following lines if we pasted at
 		// least one note...
-		eng()->getSongEditor()->setModified();
+		engine::getSongEditor()->setModified();
 		update();
-		eng()->getSongEditor()->update();
+		engine::getSongEditor()->update();
 	}
 }
 
@@ -2545,9 +2585,9 @@ void pianoRoll::deleteSelectedNotes( void )
 
 	if( update_after_delete == TRUE )
 	{
-		eng()->getSongEditor()->setModified();
+		engine::getSongEditor()->setModified();
 		update();
-		eng()->getSongEditor()->update();
+		engine::getSongEditor()->update();
 	}
 }
 
@@ -2556,8 +2596,8 @@ void pianoRoll::deleteSelectedNotes( void )
 
 void pianoRoll::updatePosition( const midiTime & _t )
 {
-	if( ( eng()->getSongEditor()->playing() &&
-			eng()->getSongEditor()->playMode() ==
+	if( ( engine::getSongEditor()->playing() &&
+			engine::getSongEditor()->playMode() ==
 					songEditor::PLAY_PATTERN ) ||
 							m_scrollBack == TRUE )
 	{
@@ -2633,12 +2673,12 @@ note * pianoRoll::noteUnderMouse( void )
 
 
 
-noteVector::iterator pianoRoll::noteIteratorUnderMouse( void )
+noteVector::const_iterator pianoRoll::noteIteratorUnderMouse( void )
 {
 	QPoint pos = mapFromGlobal( QCursor::pos() );
 
 	// get note-vector of current pattern
-	noteVector & notes = m_pattern->notes();
+	const noteVector & notes = m_pattern->notes();
 
 	if( pos.x() <= WHITE_KEY_WIDTH || pos.x() > width() - SCROLLBAR_SIZE
 		|| pos.y() < PR_TOP_MARGIN
@@ -2652,7 +2692,7 @@ noteVector::iterator pianoRoll::noteIteratorUnderMouse( void )
 							+ m_currentPosition;
 
 	// will be our iterator in the following loop
-	noteVector::iterator it = notes.begin();
+	noteVector::const_iterator it = notes.begin();
 
 	// loop through whole note-vector...
 	while( it != notes.end() )
@@ -2673,7 +2713,7 @@ noteVector::iterator pianoRoll::noteIteratorUnderMouse( void )
 
 
 
-
+#ifdef BUILD_LINUX
 bool pianoRoll::x11Event( XEvent * _xe )
 {
 	if( validPattern() )
@@ -2683,7 +2723,7 @@ bool pianoRoll::x11Event( XEvent * _xe )
 	}
 	return( FALSE );
 }
-
+#endif
 
 
 
