@@ -4,7 +4,7 @@
  * instrument_track.cpp - implementation of instrument-track-class
  *                        (window + data-structures)
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -40,6 +40,7 @@
 #include <QtGui/QLayout>
 #include <QtGui/QLineEdit>
 #include <QtGui/QMenu>
+#include <QtGui/QMessageBox>
 
 #else
 
@@ -54,39 +55,46 @@
 #include <qlabel.h>
 #include <qpopupmenu.h>
 #include <qwhatsthis.h>
+#include <qmessagebox.h>
 
 #endif
 
 
 #include "instrument_track.h"
-#include "pattern.h"
-#include "main_window.h"
-#include "song_editor.h"
-#include "effect_board.h"
-#include "envelope_tab_widget.h"
 #include "arp_and_chords_tab_widget.h"
-#include "instrument.h"
 #include "audio_port.h"
+#include "automation_pattern.h"
+#include "config_mgr.h"
+#include "debug.h"
+#include "effect_board.h"
+#include "effect_chain.h"
+#include "effect_tab_widget.h"
+#include "embed.h"
+#include "engine.h"
+#include "envelope_tab_widget.h"
+#include "fade_button.h"
+#include "gui_templates.h"
+#include "instrument.h"
+#include "lcd_spinbox.h"
+#include "led_checkbox.h"
+#include "main_window.h"
 #include "midi_client.h"
 #include "midi_port.h"
 #include "midi_tab_widget.h"
-#include "note_play_handle.h"
-#include "embed.h"
-#include "fade_button.h"
-#include "lcd_spinbox.h"
-#include "led_checkbox.h"
-#include "piano_widget.h"
-#include "surround_area.h"
-#include "tooltip.h"
-#include "tab_widget.h"
-#include "buffer_allocator.h"
-#include "config_mgr.h"
-#include "debug.h"
 #include "mmp.h"
+#include "note_play_handle.h"
+#include "pattern.h"
+#include "piano_widget.h"
+#include "sample_play_handle.h"
+#include "song_editor.h"
 #include "string_pair_drag.h"
+#include "surround_area.h"
+#include "tab_widget.h"
+#include "tooltip.h"
 #include "volume_knob.h"
 
 
+
 const char * volume_help = QT_TRANSLATE_NOOP( "instrumentTrack",
 						"With this knob you can set "
 						"the volume of the opened "
@@ -101,22 +109,20 @@ const char * surroundarea_help = QT_TRANSLATE_NOOP( "instrumentTrack",
 						"feature." );
 
 
-const int CHANNEL_WIDTH		= 250;
-const int INSTRUMENT_WIDTH	= CHANNEL_WIDTH;
+const int INSTRUMENT_WIDTH	= 250;
 const int INSTRUMENT_HEIGHT	= INSTRUMENT_WIDTH;
 const int PIANO_HEIGHT		= 84;
 
 
 instrumentTrack::instrumentTrack( trackContainer * _tc ) :
-	QWidget( _tc->eng()->getMainWindow()->workspace() ),
+	QWidget( engine::getMainWindow()->workspace() ),
  	track( _tc ),
 	midiEventProcessor(),
-	m_trackType( CHANNEL_TRACK ),
-	m_midiPort( eng()->getMixer()->getMIDIClient()->addPort( this,
+	m_trackType( INSTRUMENT_TRACK ),
+	m_midiPort( engine::getMixer()->getMIDIClient()->addPort( this,
 						tr( "unnamed_channel" ) ) ),
-	m_audioPort( new audioPort( tr( "unnamed_channel" ), eng() ) ),
+	m_audioPort( new audioPort( tr( "unnamed_channel" ) ) ),
 	m_notes(),
-	m_notesMutex(),
 	m_baseTone( A ),
 	m_baseOctave( OCTAVE_4 ),
 	m_instrument( NULL ),
@@ -128,16 +134,14 @@ instrumentTrack::instrumentTrack( trackContainer * _tc ) :
 	m_midiOutputID( -1 )
 #endif
 {
-	m_notesMutex.lock();
-	for( int i = 0; i < NOTES_PER_OCTAVE * OCTAVES; ++i )
+	for( int i = 0; i < NOTES; ++i )
 	{
 		m_notes[i] = NULL;
 	}
-	m_notesMutex.unlock();
 
 
 #ifdef QT4
-	eng()->getMainWindow()->workspace()->addWindow( this );
+	engine::getMainWindow()->workspace()->addWindow( this );
 #endif
 
 	setAcceptDrops( TRUE );
@@ -149,7 +153,7 @@ instrumentTrack::instrumentTrack( trackContainer * _tc ) :
 
 	// creation of widgets for track-settings-widget
 	m_tswVolumeKnob = new volumeKnob( knobSmall_17, getTrackSettingsWidget(),
-					tr( "Channel volume" ), eng(), this );
+						tr( "Channel volume" ), this );
 	m_tswVolumeKnob->setRange( MIN_VOLUME, MAX_VOLUME, 1.0f );
 	m_tswVolumeKnob->setInitValue( DEFAULT_VOLUME );
 	m_tswVolumeKnob->setHintText( tr( "Channel volume:" ) + " ", "%" );
@@ -220,7 +224,7 @@ instrumentTrack::instrumentTrack( trackContainer * _tc ) :
 
 	// setup volume-knob
 	m_volumeKnob = new volumeKnob( knobBright_26, m_generalSettingsWidget,
-					tr( "Channel volume" ), eng(), this );
+						tr( "Channel volume" ), this );
 	m_volumeKnob->move( 10, 44 );
 	m_volumeKnob->setRange( MIN_VOLUME, MAX_VOLUME, 1.0f );
 	m_volumeKnob->setInitValue( DEFAULT_VOLUME );
@@ -240,7 +244,7 @@ instrumentTrack::instrumentTrack( trackContainer * _tc ) :
 
 	// setup surround-area
 	m_surroundArea = new surroundArea( m_generalSettingsWidget,
-					tr( "Surround area" ), eng(), this );
+						tr( "Surround area" ), this );
 	m_surroundArea->move( 20 + m_volumeKnob->width(), 38 );
 	m_surroundArea->show();
 #ifdef QT4
@@ -261,7 +265,7 @@ instrumentTrack::instrumentTrack( trackContainer * _tc ) :
 						MAX_EFFECT_CHANNEL, 2,
 						m_generalSettingsWidget,
 						tr( "FX channel" ),
-						eng(), this );
+						this );
 	m_effectChannelNumber->setInitValue( DEFAULT_EFFECT_CHANNEL );
 	m_effectChannelNumber->setLabel( tr( "FX CHNL" ) );
 	m_effectChannelNumber->move( m_surroundArea->x() +
@@ -301,13 +305,15 @@ instrumentTrack::instrumentTrack( trackContainer * _tc ) :
 	m_envWidget = new envelopeTabWidget( this );
 	m_arpWidget = new arpAndChordsTabWidget( this );
 	m_midiWidget = new midiTabWidget( this, m_midiPort );
+	m_effWidget = new effectTabWidget( this, m_audioPort );
 	m_tabWidget->addTab( m_envWidget, tr( "ENV/LFO/FILTER" ), 1 );
 	m_tabWidget->addTab( m_arpWidget, tr( "ARP/CHORD" ), 2 );
-	m_tabWidget->addTab( m_midiWidget, tr( "MIDI" ), 3 );
+	m_tabWidget->addTab( m_effWidget, tr( "FX" ), 3 );
+	m_tabWidget->addTab( m_midiWidget, tr( "MIDI" ), 4 );
 
 	// setup piano-widget
 	m_pianoWidget = new pianoWidget( this );
-	m_pianoWidget->setFixedSize( CHANNEL_WIDTH, PIANO_HEIGHT );
+	m_pianoWidget->setFixedSize( INSTRUMENT_WIDTH, PIANO_HEIGHT );
 
 
 	vlayout->addWidget( m_generalSettingsWidget );
@@ -315,22 +321,30 @@ instrumentTrack::instrumentTrack( trackContainer * _tc ) :
 	vlayout->addWidget( m_pianoWidget );
 
 #ifdef QT4
-	m_midiInputAction = m_tswMidiMenu->addMenu(
-						m_midiWidget->m_readablePorts );
-	m_midiOutputAction = m_tswMidiMenu->addMenu(
-					m_midiWidget->m_writeablePorts );
-	m_midiInputAction->setText( tr( "MIDI input" ) );
-	m_midiOutputAction->setText( tr( "MIDI output" ) );
-	if( m_midiWidget->m_readablePorts == NULL )
+	if( m_midiWidget->m_readablePorts )
 	{
+		m_midiInputAction = m_tswMidiMenu->addMenu(
+					m_midiWidget->m_readablePorts );
+	}
+	else
+	{
+		m_midiInputAction = m_tswMidiMenu->addAction( "" );
 		connect( m_midiInputAction, SIGNAL( changed() ), this,
 						SLOT( midiInSelected() ) );
 	}
-	if( m_midiWidget->m_writeablePorts == NULL )
+	if( m_midiWidget->m_writeablePorts )
 	{
-		connect( m_midiOutputAction, SIGNAL( changed() ), this,
-						SLOT( midiOutSelected() ) );
+		m_midiOutputAction = m_tswMidiMenu->addMenu(
+					m_midiWidget->m_writeablePorts );
 	}
+	else
+	{
+		m_midiOutputAction = m_tswMidiMenu->addAction( "" );
+		connect( m_midiOutputAction, SIGNAL( changed() ), this,
+					SLOT( midiOutSelected() ) );
+	}
+	m_midiInputAction->setText( tr( "MIDI input" ) );
+	m_midiOutputAction->setText( tr( "MIDI output" ) );
 #else
 	m_midiInputID = m_tswMidiMenu->insertItem( tr( "MIDI input" ),
 					m_midiWidget->m_readablePorts );
@@ -365,9 +379,10 @@ instrumentTrack::instrumentTrack( trackContainer * _tc ) :
 
 	_tc->updateAfterTrackAdd();
 #ifndef QT3
-	setFixedWidth( CHANNEL_WIDTH );
+	setFixedWidth( INSTRUMENT_WIDTH );
 	resize( sizeHint() );
 #endif
+
 }
 
 
@@ -377,9 +392,10 @@ instrumentTrack::instrumentTrack( trackContainer * _tc ) :
 
 instrumentTrack::~instrumentTrack()
 {
-	invalidateAllMyNPH();
+	engine::getMixer()->removePlayHandles( this );
+	delete m_effWidget;
 	delete m_audioPort;
-	eng()->getMixer()->getMIDIClient()->removePort( m_midiPort );
+	engine::getMixer()->getMIDIClient()->removePort( m_midiPort );
 }
 
 
@@ -425,10 +441,11 @@ void instrumentTrack::saveSettingsBtnClicked( void )
 		saveTrackSpecificSettings( mmp, _this );
 		mmp.content().appendChild( _this );
 #ifdef QT4
-		mmp.writeFile( sfd.selectedFiles()[0] );
+		QString f = sfd.selectedFiles()[0];
 #else
-		mmp.writeFile( sfd.selectedFile() );
+		QString f = sfd.selectedFile();
 #endif
+		mmp.writeFile( f );
 	}
 }
 
@@ -506,21 +523,6 @@ void instrumentTrack::midiConfigChanged( bool )
 
 
 
-float instrumentTrack::frequency( notePlayHandle * _n ) const
-{
-	float pitch = (float)( _n->tone() - m_baseTone +
-			eng()->getSongEditor()->masterPitch() ) / 12.0f +
-					(float)( _n->octave() - m_baseOctave );
-	if( _n->detuning() )
-	{
-		pitch += _n->detuning()->value() / 12.0f;
-	}
-	return( BASE_FREQ * powf( 2.0f, pitch ) );
-}
-
-
-
-
 f_cnt_t instrumentTrack::beatLen( notePlayHandle * _n ) const
 {
 	if( m_instrument != NULL )
@@ -540,15 +542,17 @@ f_cnt_t instrumentTrack::beatLen( notePlayHandle * _n ) const
 
 
 void instrumentTrack::processAudioBuffer( sampleFrame * _buf,
-							const fpab_t _frames,
+							const fpp_t _frames,
 							notePlayHandle * _n )
 {
 	// we must not play the sound if this instrumentTrack is muted...
-	if( muted() )
+	if( muted() || ( _n && _n->bbTrackMuted() ) )
 	{
 		return;
 	}
 	float v_scale = (float) getVolume() / DEFAULT_VOLUME;
+	
+	m_audioPort->getEffects()->startRunning();
 
 	// instruments using instrument-play-handles will call this method
 	// without any knowledge about notes, so they pass NULL for _n, which
@@ -557,50 +561,45 @@ void instrumentTrack::processAudioBuffer( sampleFrame * _buf,
 	{
 		m_envWidget->processAudioBuffer( _buf, _frames, _n );
 		v_scale *= ( (float) _n->getVolume() / DEFAULT_VOLUME );
-		const fpab_t ENV_FRAMES = 32;
+/*		const fpp_t ENV_FRAMES = 10;
 		if( _n->totalFramesPlayed() == 0 )
 		{
 			// very basic envelope for not having clicks at the
 			// beginning
-			for( fpab_t i = 0; i < tMin<fpab_t>( _frames,
-							ENV_FRAMES ); ++i )
+			const fpp_t frames = tMin<fpp_t>( _frames,
+								ENV_FRAMES );
+			for( fpp_t i = 0; i < frames; ++i )
 			{
 				for( ch_cnt_t ch = 0; ch < DEFAULT_CHANNELS;
 									++ch )
 				{
-					_buf[i][ch] *= (float) i / ENV_FRAMES;
+					_buf[i][ch] *= (float) i / frames;
 				}
 			}
 		}
+
 		// last time we're called for current note?
-		if( ( _n->actualReleaseFramesToDo() == 0 &&
-			_n->totalFramesPlayed() +
-				eng()->getMixer()->framesPerAudioBuffer() >=
-							_n->frames() ) ||
-				( _n->released() && _n->releaseFramesDone() +
-			eng()->getMixer()->framesPerAudioBuffer() >=
-					_n->actualReleaseFramesToDo() ) )
+		if( _n->willFinishThisPeriod() )
 		{
 			// then do a soft fade-out at the end to avoid clicks
-			for( fpab_t i = ( _frames >= ENV_FRAMES ) ?
+			for( fpp_t i = ( _frames >= ENV_FRAMES ) ?
 						_frames - ENV_FRAMES : 0;
 							i < _frames; ++i )
 			{
 				for( ch_cnt_t ch = 0; ch < DEFAULT_CHANNELS;
 									++ch )
 				{
-					_buf[i][ch] *= (float) ( _frames - i ) /
+					_buf[i][ch] *= (float) ( _frames-i-1 ) /
 								ENV_FRAMES;
 				}
 			}
-		}
+		}*/
 	}
-
 	volumeVector v = m_surroundArea->getVolumeVector( v_scale );
 
-	eng()->getMixer()->bufferToPort( _buf, _frames,
-				( _n != NULL ) ? _n->framesAhead() : 0, v,
-								m_audioPort );
+	engine::getMixer()->bufferToPort( _buf,
+		( _n != NULL ) ? tMin<f_cnt_t>( _n->framesLeftForCurrentPeriod(), _frames ) : _frames,
+			( _n != NULL ) ? _n->offset() : 0, v, m_audioPort );
 }
 
 
@@ -609,7 +608,6 @@ void instrumentTrack::processAudioBuffer( sampleFrame * _buf,
 void instrumentTrack::processInEvent( const midiEvent & _me,
 							const midiTime & _time )
 {
-	m_notesMutex.lock();
 	switch( _me.m_type )
 	{
 		case NOTE_ON: 
@@ -631,46 +629,62 @@ void instrumentTrack::processInEvent( const midiEvent & _me,
 					// create (timed) note-play-handle
 					notePlayHandle * nph = new
 						notePlayHandle( this,
-			_time.frames( eng()->getSongEditor()->framesPerTact() ),
-						valueRanges<f_cnt_t>::max, n );
-					if( eng()->getMixer()->addPlayHandle(
+							_time.frames(
+						engine::framesPerTact64th() ),
+						valueRanges<f_cnt_t>::max / 2,
+									n );
+					if( engine::getMixer()->addPlayHandle(
 									nph ) )
 					{
 						m_notes[_me.key()] = nph;
 					}
+					return;
 				}
 				break;
 			}
 
 		case NOTE_OFF:
-			if( m_notes[_me.key()] != NULL )
+		{
+			notePlayHandle * n = m_notes[_me.key()];
+			if( n != NULL )
 			{
-				notePlayHandle * n = m_notes[_me.key()];
 				// create dummy-note which has the same length
 				// as the played note for sending it later
 				// to all slots connected to signal noteDone()
 				// this is for example needed by piano-roll for
 				// recording notes into a pattern
-				note done_note( NULL,
+				note done_note(
 					midiTime( static_cast<f_cnt_t>(
-					n->totalFramesPlayed() * 64 /
-				eng()->getSongEditor()->framesPerTact() ) ),
+						n->totalFramesPlayed() /
+						engine::framesPerTact64th() ) ),
 					0, n->tone(), n->octave(),
 					n->getVolume(), n->getPanning() );
+				// lock our play-handle while calling noteOff()
+				// for not modifying it's member variables
+				// asynchronously (while rendering!)
+				// which can result in segfaults
+				engine::getMixer()->lock();
 				n->noteOff();
+				engine::getMixer()->unlock();
 				m_notes[_me.key()] = NULL;
 				emit noteDone( done_note );
 			}
 			break;
+		}
 
 		case KEY_PRESSURE:
-			if( m_notes[_me.key()] != NULL )
+			if( !m_instrument->handleMidiEvent( _me, _time ) &&
+						m_notes[_me.key()] != NULL )
 			{
 				m_notes[_me.key()]->setVolume( _me.velocity() *
-								100 / 128 );
+								100 / 127 );
 			}
 			break;
 
+		case PITCH_BEND:
+			m_instrument->handleMidiEvent( _me, _time );
+			break;
+
 /*		case PITCH_BEND:
 			if( m_pitchBendKnob != NULL )
 			{
@@ -685,11 +699,10 @@ void instrumentTrack::processInEvent( const midiEvent & _me,
 			break;*/
 
 		default:
-			printf( "channel-track: unhandled MIDI-event %d\n",
+			printf( "instrument-track: unhandled MIDI-event %d\n",
 								_me.m_type );
 			break;
 	}
-	m_notesMutex.unlock();
 }
 
 
@@ -698,15 +711,35 @@ void instrumentTrack::processInEvent( const midiEvent & _me,
 void instrumentTrack::processOutEvent( const midiEvent & _me,
 							const midiTime & _time )
 {
-	if( _me.m_type == NOTE_ON && !configManager::inst()->value( "ui",
-				"disablechannelactivityindicators" ).toInt() )
+	switch( _me.m_type )
 	{
-		//QMutexLocker ml( &m_notesMutex );
-		if( m_notes[_me.key()] != NULL )
-		{
-			return;
-		}
-		m_tswActivityIndicator->activate();
+		case NOTE_ON:
+			if( !configManager::inst()->value( "ui",
+						"manualchannelpiano" ).toInt() )
+			{
+				m_pianoWidget->setKeyState( _me.key(), TRUE );
+			}
+			if( !configManager::inst()->value( "ui",
+				"disablechannelactivityindicators" ).toInt() )
+			{
+				if( m_notes[_me.key()] != NULL )
+				{
+					return;
+				}
+				m_tswActivityIndicator->activate();
+			}
+			break;
+
+		case NOTE_OFF:
+			if( !configManager::inst()->value( "ui",
+						"manualchannelpiano" ).toInt() )
+			{
+				m_pianoWidget->setKeyState( _me.key(), FALSE );
+			}
+			break;
+
+		default:
+			break;
 	}
 	// if appropriate, midi-port does futher routing
 	m_midiPort->processOutEvent( _me, _time );
@@ -715,7 +748,7 @@ void instrumentTrack::processOutEvent( const midiEvent & _me,
 
 
 
-void instrumentTrack::playNote( notePlayHandle * _n )
+void instrumentTrack::playNote( notePlayHandle * _n, bool _try_parallelizing )
 {
 	// arpeggio- and chord-widget has to do its work -> adding sub-notes
 	// for chords/arpeggios
@@ -723,8 +756,68 @@ void instrumentTrack::playNote( notePlayHandle * _n )
 
 	if( _n->arpBaseNote() == FALSE && m_instrument != NULL )
 	{
+		if( m_instrument->isMonophonic() )
+		{
+			constNotePlayHandleVector v =
+				notePlayHandle::nphsOfInstrumentTrack( this,
+									TRUE );
+			if( v.size() > 1 )
+			{
+				constNotePlayHandleVector::iterator
+						youngest_note = v.begin();
+				for( constNotePlayHandleVector::iterator it =
+						v.begin(); it != v.end(); ++it )
+				{
+					if( !( *it )->arpBaseNote() &&
+						( *it )->totalFramesPlayed() <=
+						( *youngest_note )->
+							totalFramesPlayed() )
+					{
+						youngest_note = it;
+					}
+				}
+				if( *youngest_note != _n &&
+					!( *youngest_note )->arpBaseNote() &&
+					!_n->released() )
+				{
+					processInEvent( midiEvent( NOTE_OFF, 0,
+						_n->key(), 0 ), midiTime() );
+					//printf("%d\n",( *youngest_note )->offset()+m_instrument->desiredTransitionFrames());
+					//_n->noteOff( ( *youngest_note )->offset()+m_instrument->desiredTransitionFrames() );
+					if( ( *youngest_note )->offset() >
+								_n->offset() )
+					{
+						_n->noteOff( ( *youngest_note )->
+								offset() -
+								_n->offset() );
+					}
+					else
+					{
+						// for the case the youngest
+						// note has an offset smaller
+						// then the current note we
+						// already played everything
+						// in last period and have
+						// to clear parts of it
+						_n->noteOff();
+	engine::getMixer()->clearAudioBuffer( m_audioPort->firstBuffer(),
+				engine::getMixer()->framesPerPeriod() -
+					( *youngest_note )->offset(),
+					( *youngest_note )->offset() );
+						return;
+					}
+				}
+				// we do not play previously released notes
+				// anymore
+				else if( *youngest_note != _n &&
+							_n->released() )
+				{
+					//return;
+				}
+			}
+		}
 		// all is done, so now lets play the note!
-		m_instrument->playNote( _n );
+		m_instrument->playNote( _n, _try_parallelizing );
 	}
 }
 
@@ -745,25 +838,23 @@ QString instrumentTrack::instrumentName( void ) const
 
 void instrumentTrack::deleteNotePluginData( notePlayHandle * _n )
 {
-	if( m_instrument != NULL )
+	if( m_instrument != NULL && _n->m_pluginData != NULL )
 	{
 		m_instrument->deleteNotePluginData( _n );
 	}
 
-	m_notesMutex.lock();
 	// Notes deleted when keys still pressed
 	if( m_notes[_n->key()] == _n )
 	{
-		note done_note( NULL, midiTime( static_cast<f_cnt_t>(
-					_n->totalFramesPlayed() * 64 /
-				eng()->getSongEditor()->framesPerTact() ) ),
+		note done_note( midiTime( static_cast<f_cnt_t>(
+						_n->totalFramesPlayed() /
+						engine::framesPerTact64th() ) ),
 					0, _n->tone(), _n->octave(),
 					_n->getVolume(), _n->getPanning() );
 		_n->noteOff();
 		m_notes[_n->key()] = NULL;
 		emit noteDone( done_note );
 	}
-	m_notesMutex.unlock();
 }
 
 
@@ -837,13 +928,21 @@ void instrumentTrack::setSurroundAreaPos( const QPoint & _p )
 
 void instrumentTrack::setBaseNote( Uint32 _new_note, bool _modified )
 {
+	engine::getMixer()->lock();
 	setBaseTone( (tones)( _new_note % NOTES_PER_OCTAVE ) );
 	setBaseOctave( (octaves)( _new_note / NOTES_PER_OCTAVE ) );
+
+	for( vlist<notePlayHandle *>::iterator it = m_processHandles.begin();
+					it != m_processHandles.end(); ++it )
+	{
+		( *it )->updateFrequency();
+	}
+	engine::getMixer()->unlock();
+
 	if( _modified )
 	{
-		eng()->getSongEditor()->setModified();
+		engine::getSongEditor()->setModified();
 	}
-	emit baseNoteChanged();
 }
 
 
@@ -874,40 +973,49 @@ void instrumentTrack::setBaseOctave( octaves _new_octave )
 int instrumentTrack::masterKey( notePlayHandle * _n ) const
 {
 	int key = baseTone() + baseOctave() * NOTES_PER_OCTAVE +
-					eng()->getSongEditor()->masterPitch();
-	return( _n->key() - ( key - A - DEFAULT_OCTAVE * NOTES_PER_OCTAVE ) );
+					engine::getSongEditor()->masterPitch();
+	return( tLimit<int>( _n->key() -
+		( key - A - DEFAULT_OCTAVE * NOTES_PER_OCTAVE ), 0, NOTES ) );
 }
 
 
 
 
 bool FASTCALL instrumentTrack::play( const midiTime & _start,
-					const f_cnt_t _start_frame,
-					const fpab_t _frames,
-					const f_cnt_t _frame_base,
+					const fpp_t _frames,
+					const f_cnt_t _offset,
 							Sint16 _tco_num )
 {
-	if( sendMidiTime( _start ) )
-	{
-		emit sentMidiTime( _start );
-	}
-
-	// calculate samples per tact; need that later when calculating
-	// sample-pos of a note
-	float frames_per_tact = eng()->getSongEditor()->framesPerTact();
+	float frames_per_tact64th = engine::framesPerTact64th();
 
 	vlist<trackContentObject *> tcos;
+	bbTrack * bb_track;
 	if( _tco_num >= 0 )
 	{
-		tcos.push_back( getTCO( _tco_num ) );
+		trackContentObject * tco = getTCO( _tco_num );
+		tcos.push_back( tco );
+		bb_track = bbTrack::findBBTrack( _tco_num );
+		if( !( ( bb_track && bb_track->automationDisabled( this ) )
+				|| dynamic_cast<pattern *>( tco )->empty() ) )
+		{
+			sendMidiTime( _start );
+		}
 	}
 	else
 	{
-		getTCOsInRange( tcos, _start, _start +
-				static_cast<Sint32>( _frames * 64 /
-							frames_per_tact ) );
+		getTCOsInRange( tcos, _start, _start + static_cast<Sint32>(
+					_frames / frames_per_tact64th ) );
+		bb_track = NULL;
+		sendMidiTime( _start );
 	}
-	
+
+	// Handle automation: detuning
+	for( vlist<notePlayHandle *>::iterator it = m_processHandles.begin();
+					it != m_processHandles.end(); ++it )
+	{
+		( *it )->processMidiTime( _start );
+	}
+
 	if ( tcos.size() == 0 )
 	{
 		return( FALSE );
@@ -915,9 +1023,6 @@ bool FASTCALL instrumentTrack::play( const midiTime & _start,
 
 	bool played_a_note = FALSE;	// will be return variable
 
-	// calculate the end of the current sample-frame
-	const f_cnt_t end_frame = _start_frame+_frames-1;
-
 	for( vlist<trackContentObject *>::iterator it = tcos.begin();
 							it != tcos.end(); ++it )
 	{
@@ -927,43 +1032,35 @@ bool FASTCALL instrumentTrack::play( const midiTime & _start,
 		{
 			continue;
 		}
-		midiTime cur_start = _start.getTact() * 64 -
-					( ( _tco_num < 0 ) ?
-						p->startPosition() :
-							midiTime( 0 ) );
-		if( p->frozen() &&
-				eng()->getSongEditor()->exporting() == FALSE )
+		midiTime cur_start = _start;
+		if( _tco_num < 0 )
 		{
-			volumeVector v = m_surroundArea->getVolumeVector(
-									1.0f );
-			// volume-vector was already used when freezing
-			// pattern, but only in stereo-mode, so front speakers
-			// are already setup
-			v.vol[0] = 1.0f;
-			v.vol[1] = 1.0f;
-			sampleFrame * buf = bufferAllocator::alloc<sampleFrame>(
-								_frames );
+			cur_start -= p->startPosition();
+		}
+		if( p->frozen() &&
+				engine::getSongEditor()->exporting() == FALSE )
+		{
+			if( cur_start > 0 )
+			{
+				continue;
+			}
 
-			p->playFrozenData( buf, _start_frame +
-						static_cast<f_cnt_t>(
-							cur_start.getTact() *
-							frames_per_tact ),
-						_frames );
-			eng()->getMixer()->bufferToPort( buf, _frames,
-								_frame_base +
-				static_cast<f_cnt_t>(
-					p->startPosition().getTact64th() *
-							frames_per_tact /
-								64.0f ),
-							v, m_audioPort );
-			bufferAllocator::free( buf );
+			samplePlayHandle * handle = new samplePlayHandle( p );
+			handle->setBBTrack( bb_track );
+			handle->setOffset( _offset );
+			// send it to the mixer
+			engine::getMixer()->addPlayHandle( handle );
+			played_a_note = TRUE;
 			continue;
 		}
 
 		// get all notes from the given pattern...
-		noteVector & notes = p->notes();
+		const noteVector & notes = p->notes();
 		// ...and set our index to zero
-		noteVector::iterator it = notes.begin();
+		noteVector::const_iterator it = notes.begin();
+#if SINGERBOT_SUPPORT
+		int note_idx = 0;
+#endif
 
 		// very effective algorithm for playing notes that are
 		// posated within the current sample-frame
@@ -974,32 +1071,25 @@ bool FASTCALL instrumentTrack::play( const midiTime & _start,
 			// skip notes which are posated before start-tact
 			while( it != notes.end() && ( *it )->pos() < cur_start )
 			{
+#if SINGERBOT_SUPPORT
+				if( ( *it )->length() != 0 )
+				{
+					++note_idx;
+				}
+#endif
 				++it;
 			}
 		}
 
-		// skip notes before sample-frame
-		while( it != notes.end() &&
-			( *it )->pos( cur_start ).frames( frames_per_tact ) <
-								_start_frame )
-		{
-			++it;
-		}
-
 		note * cur_note;
 		while( it != notes.end() &&
-			( ( cur_note = *it )->pos( cur_start ).frames(
-					frames_per_tact ) ) <= end_frame )
+					( cur_note = *it )->pos() == cur_start )
 		{
 			if( cur_note->length() != 0 )
 			{
-				const f_cnt_t frames_ahead = _frame_base -
-								_start_frame +
-			cur_note->pos( cur_start ).frames( frames_per_tact );
-
 				const f_cnt_t note_frames =
 					cur_note->length().frames(
-							frames_per_tact );
+							frames_per_tact64th );
 
 /*				// generate according MIDI-events
 				processOutEvent( midiEvent( NOTE_ON,
@@ -1008,38 +1098,32 @@ bool FASTCALL instrumentTrack::play( const midiTime & _start,
 							cur_note->getVolume() *
 								128 / 100 ),
 						midiTime::fromFrames(
-							frames_ahead,
-							frames_per_tact ) );
+							_offset,
+							frames_per_tact64th ) );
 
 				processOutEvent( midiEvent( NOTE_OFF,
 						m_midiPort->outputChannel(),
 							cur_note->key(), 0 ),
 						midiTime::fromFrames(
-							frames_ahead +
+							_offset+
 								note_frames,
-							frames_per_tact ) );*/
+							frames_per_tact64th ) );
+*/
 
 				notePlayHandle * note_play_handle =
-					new notePlayHandle( this,
-								frames_ahead,
+					new notePlayHandle( this, _offset,
 								note_frames,
 								*cur_note );
-				note_play_handle->play();
-				// could we play all within current number of
-				// frames per audio-buffer?
-				if( note_play_handle->done() == FALSE )
-				{
-					// no, then insert it into
-					// play-handle-vector of mixer
-					eng()->getMixer()->addPlayHandle(
+				note_play_handle->setBBTrack( bb_track );
+#if SINGERBOT_SUPPORT
+				note_play_handle->setPatternIndex( note_idx );
+#endif
+				engine::getMixer()->addPlayHandle(
 							note_play_handle );
-				}
-				else
-				{
-					// otherwise just throw it away...
-					delete note_play_handle;
-				}
 				played_a_note = TRUE;
+#if SINGERBOT_SUPPORT
+				++note_idx;
+#endif
 			}
 			++it;
 		}
@@ -1079,6 +1163,7 @@ void instrumentTrack::saveTrackSpecificSettings( QDomDocument & _doc,
 	m_envWidget->saveState( _doc, _this );
 	m_arpWidget->saveState( _doc, _this );
 	m_midiWidget->saveState( _doc, _this );
+	m_effWidget->saveState( _doc, _this );
 }
 
 
@@ -1088,6 +1173,7 @@ void instrumentTrack::loadTrackSpecificSettings( const QDomElement & _this )
 {
 	invalidateAllMyNPH();
 
+	engine::getMixer()->lock();
 	setName( _this.attribute( "name" ) );
 	m_volumeKnob->loadSettings( _this, "vol" );
 
@@ -1097,6 +1183,8 @@ void instrumentTrack::loadTrackSpecificSettings( const QDomElement & _this )
 	m_pianoWidget->loadSettings( _this, "basenote" );
 	int tab = _this.attribute( "tab" ).toInt();
 
+	bool had_fx = FALSE;
+
 	QDomNode node = _this.firstChild();
 	while( !node.isNull() )
 	{
@@ -1114,6 +1202,11 @@ void instrumentTrack::loadTrackSpecificSettings( const QDomElement & _this )
 			{
 				m_midiWidget->restoreState( node.toElement() );
 			}
+			else if( m_effWidget->nodeName() == node.nodeName() )
+			{
+				m_effWidget->restoreState( node.toElement() );
+				had_fx = TRUE;
+			}
 			else if( automationPattern::classNodeName()
 							!= node.nodeName() )
 			{
@@ -1135,6 +1228,11 @@ void instrumentTrack::loadTrackSpecificSettings( const QDomElement & _this )
 		}
 		node = node.nextSibling();
         }
+	if( !had_fx )
+	{
+		m_effWidget->deleteAllEffects();
+	}
+	engine::getMixer()->unlock();
 
 	m_tabWidget->setActiveTab( tab );
 
@@ -1154,8 +1252,10 @@ instrument * instrumentTrack::loadInstrument( const QString & _plugin_name )
 {
 	invalidateAllMyNPH();
 
+	engine::getMixer()->lock();
 	delete m_instrument;
 	m_instrument = instrument::instantiate( _plugin_name, this );
+	engine::getMixer()->unlock();
 
 	m_tabWidget->addTab( m_instrument, tr( "PLUGIN" ), 0 );
 	m_tabWidget->setActiveTab( 0 );
@@ -1179,7 +1279,7 @@ instrument * instrumentTrack::loadInstrument( const QString & _plugin_name )
 void instrumentTrack::surroundAreaPosChanged( const QPoint & _p )
 {
 	setSurroundAreaPos( _p );
-	eng()->getSongEditor()->setModified();
+	engine::getSongEditor()->setModified();
 }
 
 
@@ -1188,7 +1288,7 @@ void instrumentTrack::surroundAreaPosChanged( const QPoint & _p )
 void instrumentTrack::textChanged( const QString & _new_name )
 {
 	setName( _new_name );
-	eng()->getSongEditor()->setModified();
+	engine::getSongEditor()->setModified();
 }
 
 
@@ -1247,7 +1347,7 @@ void instrumentTrack::dropEvent( QDropEvent * _de )
 	if( type == "instrument" )
 	{
 		loadInstrument( value );
-		eng()->getSongEditor()->setModified();
+		engine::getSongEditor()->setModified();
 		_de->accept();
 	}
 	else if( type == "presetfile" )
@@ -1255,7 +1355,7 @@ void instrumentTrack::dropEvent( QDropEvent * _de )
 		multimediaProject mmp( value );
 		loadTrackSpecificSettings( mmp.content().firstChild().
 								toElement() );
-		eng()->getSongEditor()->setModified();
+		engine::getSongEditor()->setModified();
 		_de->accept();
 	}
 }
@@ -1265,22 +1365,16 @@ void instrumentTrack::dropEvent( QDropEvent * _de )
 
 void instrumentTrack::invalidateAllMyNPH( void )
 {
-	// note-play-handles check track-type to determine whether their
-	// channel-track is being deleted (if this is the case, they
-	// invalidate themselves)
-	m_trackType = NULL_TRACK;
-
-	m_notesMutex.lock();
-	for( int i = 0; i < NOTES_PER_OCTAVE * OCTAVES; ++i )
+	engine::getMixer()->lock();
+	for( int i = 0; i < NOTES; ++i )
 	{
 		m_notes[i] = NULL;
 	}
-	m_notesMutex.unlock();
 
 	// invalidate all note-play-handles linked to this channel
-	eng()->getMixer()->checkValidityOfPlayHandles();
-
-	m_trackType = CHANNEL_TRACK;
+	m_processHandles.clear();
+	engine::getMixer()->removePlayHandles( this );
+	engine::getMixer()->unlock();
 }
 
 
